<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 08.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-11-07 20:13:01 +0100 (Mo, 07. Nov 2011) $
 * last changed by: $LastChangedBy: sni $
 * revision       : $LastChangedRevision: 8947 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/ReasonEditor/trunk/hpls.hmt.reason.editor.gui.frontend/application/modules/hmtreason/controllers/EditController.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Mailtrace/Controller/Action.php';
require_once 'Hmt/Reason/Edit.php';
/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 8947 $
 * @since 1.0.9
 */
class HmtReason_EditController extends Hpls_Mailtrace_Controller_Action {

	public function preDispatch() {
		if (null != $this->_getParam(Hpls_Consts::FORM_SUBMIT_CANCEL, null)) {
			if (null != $this->_getParam(Hpls_Consts::FORM_REDIRECT, null)) {
				$this->_redirect($this->_getParam(Hpls_Consts::FORM_REDIRECT, null));
			} else {
				$this->_redirect("hmtreason/edit");
			}
		}
	}

	public function indexAction() {
		$this->log()->debug(__METHOD__);
		/**
		 * @var Hmt_Reason_Edit
		 */
		$hre = Hmt_Reason_Edit::i();
		$hre->setCurrentTarget(Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($hre->getRules()));
		$this->view->rules = Hmt_Reason_Edit::i()->getRules();
		$this->view->hmtr_db_options = array(
				'vendor' => 'Hersteller',
				'customer' => 'Kunde' );
	}

	public function ruleAction() {
		if (NULL != $this->_getParam(Hpls_Consts::FORM_SUBMIT_SAVE, null)) {
			//formular abgeschickt -> save
			$rule = $this->saveRule();
			Hmt_Reason_Edit::i()->clearCache();
			$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $rule->getID());
		}
		//ab hier aufbereitung für anzeige
		$default = (hasRole(Hpls_Acl::ROLE_HPLSADMIN) ? Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB : Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB);
		$oldTarget = $this->_getParam('hmtr_db', $default);
		Hmt_Reason_Edit::i()->setCurrentTarget(Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
		$action = $this->_getParam('do', 'edit');
		$ruleId = $this->_getParam('ruleid', null);
		if ($action == 'edit' || $action == 'add') {
			$source = array();
			if (FALSE === is_null($ruleId)) {
				//edit
				$i = Hmt_Reason_Edit::i()->getRule($ruleId);
				foreach (array(
						Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB,
						Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB,
						Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB ) as $sourcedb) {
					Hmt_Reason_Edit::i()->setCurrentTarget($sourcedb);
					$source[$sourcedb] = array(
							'regexs' => Hmt_Reason_Edit::i()->getRegExs($ruleId),
							'rtexts' => Hmt_Reason_Edit::i()->getReasonText($ruleId),
							'roles' => Hmt_Reason_Edit::i()->getRoles(),
							'availLangs' => Hmt_Reason_Edit::i()->getAvaiableLanguages() );
				}
				//dirty hack: die customer db hat keine rolen, aber für die ausgabe werden welche benötigt -> fremdkopieren
				Hmt_Reason_Edit::i()->setCurrentTarget(Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
				$source[Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB]['roles'] = Hmt_Reason_Edit::i()->getRoles();
				$this->view->rule = $i[0];
				$this->view->isNewEntry = false;
				$this->view->source = $source;
			} else {
				//add
				$this->view->isNewEntry = true;
			}
			$this->view->hmtr_db = Hmt_Reason_Edit::i()->getCurrentTarget();
			$this->view->hmtr_db_options = array(
					'vendor' => 'Hersteller',
					'customer' => 'Kunde' );
		} elseif ($action == 'del') {
			try {
				//				$this->log()->debug(__METHOD__ . ' [SNi3] ' . $oldTarget);
				Hmt_Reason_Edit::i()->delRule($ruleId, $oldTarget);
			} catch (Exception $ex) {
				$this->log()->err($ex);
			}
			$this->_redirect("hmtreason/edit");
		}
	}

	public function regexAction() {
		if (NULL != $this->_getParam(Hpls_Consts::FORM_SUBMIT_SAVE, null)) {
			try {
				$regex = $this->saveRegEx();
				Hmt_Reason_Edit::i()->clearCache();
				$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $regex->getRuleId());
			} catch (Exception $ex) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'regex', $ex->getMessage(), $ex->getCode());
			}
		
		//			$this->_setParam('regexid', $regex->getID());
		}
		$default = (hasRole(Hpls_Acl::ROLE_HPLSADMIN) ? Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB : Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB);
		$oldTarget = $this->_getParam('hmtr_db', $default);
		Hmt_Reason_Edit::i()->setCurrentTarget(Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
		$action = $this->_getParam('do', 'edit');
		$regexid = $this->_getParam('regexid', 0);
		if ($action == 'edit' || $action == 'add') {
			$rules = Hmt_Reason_Edit::i()->getRules();
			$this->view->rules = $rules;
			if ($regexid > 0) {
				$i = Hmt_Reason_Edit::i()->getRegEx($regexid);
				$this->view->regex = $i[0];
				$this->view->redirectUrl = '/hmtreason/edit/rule/do/edit/ruleid/' . $i[0]->getRuleId();
				$this->view->isNewEntry = false;
			} else {
				$entry = new Hmt_Reason_Edit_RegEx();
				$entry->setRuleId($this->_getParam('ruleid', null));
				$entry->setPriority(0);
				$entry->setDeactivated(false);
				$entry->setRegEx('//i');
				$this->view->regex = $entry;
				$this->view->isNewEntry = true;
				if (null != $this->_getParam('ruleid', null)) {
					$this->view->redirectUrl = '/hmtreason/edit/rule/do/edit/ruleid/' . $this->_getParam('ruleid', null);
				} else {
					$this->view->redirectUrl = '/hmtreason/edit';
				}
			}
			$this->view->hmtr_db = Hmt_Reason_Edit::i()->getCurrentTarget();
			$this->view->hmtr_db_options = array(
					'vendor' => 'Hersteller',
					'customer' => 'Kunde' );
		} elseif ($action == 'del') {
			if ($regexid > 0) {
				try {
					$i = Hmt_Reason_Edit::i()->getRegEx($regexid); // nur für den späteren redirect
					$regex = $i[0];
					Hmt_Reason_Edit::i()->delRegEx($regexid, $oldTarget);
					$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $regex->getRuleId());
				} catch (Exception $ex) {
					$this->log()->err($ex);
				}
			}
		}
	}

	public function textAction() {
		if (NULL != $this->_getParam(Hpls_Consts::FORM_SUBMIT_SAVE, null)) {
			$rtext = $this->saveRText();
			Hmt_Reason_Edit::i()->clearCache();
			$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $rtext->getRuleId());
		
		//			$this->_setParam('textid', $regex->getID());
		//			$this->_getParam('lang', $regex->getLanguage());
		}
		$action = $this->_getParam('do', 'add');
		$textId = $this->_getParam('textid', 0);
		$lang = $this->_getParam('lang', 'de_de');
		Hmt_Reason_Edit::i()->setCurrentTarget(Hpls_Mailtrace_Consts::CONFIG_KEY_CACHEDB);
		if ($action == 'edit' || $action == 'add') {
			$this->view->roles = Hmt_Reason_Edit::i()->getRoles();
			$l = Hmt_Reason_Edit::i()->getAvaiableLanguages();
			$this->view->languages = array_combine($l, $l);
			$this->view->rules = Hmt_Reason_Edit::i()->getRules();
			if ($textId > 0) {
				//edit entry
				$this->view->isNewEntry = false;
				//			$this->view->rule = $i = Hmt_Reason_Edit::i()->getRule($textId);
				$i = Hmt_Reason_Edit::i()->getReasonTextById($lang, $textId);
				if (count($i) < 1) {
					$this->_forward('error', 'index', 'index');
					return;
				}
				$this->view->rtext = $i[0];
				$this->view->redirectUrl = 'hmtreason/edit/rule/do/edit/ruleid/' . $i[0]->getRuleId();
			} else {
				//add entry
				$entry = new Hmt_Reason_Edit_Text();
				$entry->setLanguage($this->_getParam('lang', Hmt_Reason_Edit::i()->getDefaultLanguage()));
				$entry->setRoleId($this->_getParam('roleid', Hmt_Reason_Edit::i()->getDefaultRole()));
				$entry->setRuleId($this->_getParam('ruleid', null));
				$this->view->rtext = $entry;
				$this->view->isNewEntry = true;
				if (null != $this->_getParam('ruleid', null)) {
					$this->view->redirectUrl = 'hmtreason/edit/rule/do/edit/ruleid/' . $this->_getParam('ruleid', null);
				} else {
					$this->view->redirectUrl = '/hmtreason/edit';
				}
			}
			$this->view->hmtr_db = Hmt_Reason_Edit::i()->getCurrentTarget();
			$this->view->hmtr_db_options = array(
					'vendor' => 'Hersteller',
					'customer' => 'Kunde' );
		} elseif ($action == 'del') {
			$lang = $this->_getParam('lang', null);
			if (!in_array($lang, Hmt_Reason_Edit::i()->getAvaiableLanguages())) {
				$this->log()->err("Language '$lang' existiert nicht, löschen von '$textId' nicht möglich.");
				$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $this->_getParam('ruleid', null));
			}
			$i = Hmt_Reason_Edit::i()->getReasonTextById($lang, $textId);
			$regex = $i[0];
			try {
				Hmt_Reason_Edit::i()->delText($lang, $textId);
			} catch (Exception $ex) {
				$this->log()->err($ex);
			}
			$this->_redirect("hmtreason/edit/rule/do/edit/ruleid/" . $regex->getRuleId());
		}
	}

	protected function saveRText() {
		$this->log()->info(__METHOD__ . '### Saving RTEXT');
		require_once 'Hmt/Reason/Edit/Text.php';
		$regex = new Hmt_Reason_Edit_Text();
		if (!Hpls_Utils::toBool($this->_getParam('hmtr_isnewentry', 0)) && !is_null($this->_getParam('hmtr_rt_id', null))) {
			$regex->setID($this->_getParam('hmtr_rt_id', null));
		}
		$regex->setLanguage($this->_getParam('hmtr_rt_lang', false));
		$regex->setRoleId($this->_getParam('hmtr_rt_roleid', false));
		$regex->setRuleId($this->_getParam('hmtr_rt_ruleid', null));
		$regex->setTextLong($this->_getParam('hmtr_rt_long', null));
		$regex->setTextShort($this->_getParam('hmtr_rt_short', null));
		$regex->setTextTag($this->_getParam('hmtr_rt_tag', null));
		//param_map
		//hmtr_re_param_map
		try {
			$this->log()->debug(Hpls_Debug::print_r($regex, __METHOD__));
			$dbName = $this->_getParam('hmtr_db', hasRole(Hpls_Acl::ROLE_HPLSADMIN) ? Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB : Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB);
			Hmt_Reason_Edit::i()->setCurrentTarget($dbName);
			if (0 <= Hmt_Reason_Edit::i()->saveText($regex)) {
				$this->view->addErrorText('hmtr', 'hmtr_text', 'Irgendwas ging schief');
				$this->log()->err(__METHOD__ . ' irgendwas ging schief');
			}
		} catch (Exception $ex) {
			$this->log()->err($ex);
		}
		return $regex;
	}

	protected function saveRule() {
		$this->log()->info(__METHOD__ . '### Saving RULE');
		require_once 'Hmt/Reason/Edit/Rule.php';
		$rule = new Hmt_Reason_Edit_Rule();
		if (FALSE === Hpls_Utils::toBool($this->_getParam('hmtr_isnewentry', false)) && FALSE === is_null($this->_getParam('hmtr_rule_id', null))) {
			$rule->setID($this->_getParam('hmtr_rule_id', null));
		}
		$rule->setName($this->_getParam('hmtr_name', null));
		$rule->setDescription($this->_getParam('hmtr_description', null));
		$rule->setDeactivated($this->_getParam('hmtr_rule_deactive', false));
		$this->log()->debug(Hpls_Debug::print_r($rule, __METHOD__ . ' SNi38'));
		try {
			$dbName = $this->_getParam('hmtr_db', hasRole(Hpls_Acl::ROLE_HPLSADMIN) ? Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB : Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB);
			Hmt_Reason_Edit::i()->setCurrentTarget($dbName);
			$this->log()->debug(Hpls_Debug::print_r($rule, __METHOD__));
			if (0 <= Hmt_Reason_Edit::i()->saveRule($rule)) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmtr_rule', 'Irgendwas ging schief');
				$this->log()->err(__METHOD__ . ' irgendwas ging schief');
			}
		} catch (Exception $ex) {
			$this->log()->err($ex);
		}
		return $rule;
	}

	/**
	 * @todo comment
	 * @return Hmt_Reason_Edit_RegEx
	 * @throws Hpls_Exception_Unify
	 */
	protected function saveRegEx() {
		$this->log()->info(__METHOD__ . '### Saving RegEx');
		require_once 'Hmt/Reason/Edit/RegEx.php';
		$regex = new Hmt_Reason_Edit_RegEx();
		if (FALSE === Hpls_Utils::toBool($this->_getParam('hmtr_isnewentry', 0)) && FALSE === is_null($this->_getParam('hmtr_re_id', null))) {
			$regex->setID($this->_getParam('hmtr_re_id', null));
		}
		$regex->setDeactivated(Hpls_Utils::toBool($this->_getParam('hmtr_re_deactive', false)));
		$regex->setDescription($this->_getParam('hmtr_re_description', null));
		$regex->setPriority($this->_getParam('hmtr_re_priority', null));
		$p_map = $this->_getParam('hmtr_re_param_map', null);
		$regex->setParamMap($p_map);
		$regex->setRuleId($this->_getParam('hmtr_re_rule', null));
		$regex->setRegEx($this->_getParam('hmtr_re_regex', null));
		//param_map
		//hmtr_re_param_map
		try {
			$this->log()->debug(Hpls_Debug::print_r($regex, __METHOD__));
			$dbName = $this->_getParam('hmtr_db', hasRole(Hpls_Acl::ROLE_HPLSADMIN) ? Hpls_Mailtrace_Consts::CONFIG_KEY_VENDORDB : Hpls_Mailtrace_Consts::CONFIG_KEY_CUSTORMERDB);
			Hmt_Reason_Edit::i()->setCurrentTarget($dbName);
			if (0 <= Hmt_Reason_Edit::i()->saveRegEx($regex)) {
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'hmtr', 'Irgendwas ging schief');
				$this->log()->err(__METHOD__ . ' irgendwas ging schief');
			}
		} catch (Exception $ex) {
			$this->log()->err($ex);
		}
		return $regex;
	}
}