<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Feb 28, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-10-23 10:03:47 +0200 (Di, 23. Okt 2012) $
 * last changed by: $LastChangedBy: d.mohn $
 * revision       : $LastChangedRevision: 9789 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.frontend/application/modules/mailtrace/controllers/ExplainController.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/

require_once 'Hpls/Mailtrace/Consts.php';
require_once 'Hpls/Mailtrace/Controller/Action.php';
require_once 'Hmt/Reason.php';
require_once 'Hmt/Log.php';
require_once 'Hmt/Db/Query/Delivers.php';
require_once 'Hmt/Db/Query/Connects.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9789 $
 * @since alpha.1
 *
 * tags
 */
class Mailtrace_ExplainController extends Hpls_Mailtrace_Controller_Action {
	protected $_mtLog = null;
	
	protected $_deliverObj = null;
	protected $_connectObj = null;

	public function preDispatch() {
// 		if (FALSE === Hpls_Application::i()->isStandAlone()) {
// 			$this->_redirect('overview');
// 		}
	}

	public function init() {
		/* Initialize action controller here */
		$this->view->assign(HVConsts::V_PAGE_TITLE, 'hmt_page_title');
	}

	/**
	 * The default action - show the home page
	 */
	public function indexAction() { // TODO Auto-generated ExplainController::indexAction() default action
		$this->_forward('show');
	}

	public function showAction() {
		
		$connect = $this->getConnect();
		$deliver = $this->getDeliver();
		isDebug() && $this->getLogger()->debug(Hpls_Debug::print_r($connect, "connect entry: "));
		isDebug() && $this->getLogger()->debug(Hpls_Debug::print_r($deliver, "deliver entry: "));
		
		$this->view->connect = $connect;
		$this->view->deliver = $deliver;
		
		/**
		 * @var Hmt_Reason_Item
		 */
		$reason = Hmt_Reason::i()->matchRule($deliver);
		if (is_null($reason)) {
			$this->_forward('request');
			return;
		}
		$reason->setConnect($connect);
		
		isDebug() && $this->getLogger()->debug(Hpls_Debug::print_r($reason, "reason entry: "));
		$this->view->reason = $reason;

		//		$this->view->assign(HVConsts::V_PAGE_SUBTITLE, 'hmt_explain_subtitle');
	

	}

	public function errorAction() {
	
	}

	public function sendmailAction(){
		$this->getHelper('layout')->disableLayout();
		$this->getHelper('ViewRenderer')->setNoRender();

		$data = $this->_request->getParams();
		// var_dump($data);
		$error = '';
		if($data['from'] == ''){
			$error = 'from';
		} else {
			$sender = $data['from'];
		}
		if($data['to'] == ''){
			if($error == 'from'){
				$error = 'both';
			} else {
				$error = 'to';
			}
		} else {
			$recipient = $data['to'];
		}
		if($error == ''){
			$message = 'Nachricht vom User: ' . $data['message'];
			
			$msgbody = $data['pagemsg'];
			$msgbody = trim(strip_tags($msgbody));
			
			$msgbody = $message . "\n" . $msgbody;
			
			$subject = 'Mailstatus Benachrichtigung';
			
			if(function_exists('mb_encode_mimeheader') && function_exists('mb_internal_encoding')){
				mb_internal_encoding("UTF-8");
				mb_encode_mimeheader($msgbody, "UTF-8", "Q");
			}
			try {
				require_once 'Hpls/Utils/Mail.php';
				Hpls_Utils_Mail::sendMail($recipient, $subject, $msgbody, $sender);
				echo 'all good';
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_INFO, 'mail', 'hmt_reason_request_send');
			} catch (Exception $ex) {
				echo 'mail not sent';
				Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'mail', $ex->getMessage(), $ex->getCode());
			}
		} else {
			echo $error;
		}
	}
	
	public function requestAction() {
		$deliver = $this->getDeliver();
		$connect = $this->getConnect();
		$this->view->connect = $connect;
		$this->view->deliver = $deliver;
		if ($this->getRequest()->isPost() && $this->getRequest()->has('save')) {
			$this->log()->debug(Hpls_Debug::print_r($this->getRequest()->getParams(), __METHOD__));
			$accepted = $this->getRequest()->getParam('hmt_accept_send', null);
			if ($accepted == 'yes') {
				$cfg = Hpls_Settings::i()->getConfig('ac.mail');
				if (empty($cfg)) {
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'mail', 'hmt_error_missing_smtp_config', Hpls_Consts0::ERR_UTILS_MAIL_NOMTA);
					return;
				}
				//mail versenden
				$recipient = $cfg->address->reason_request;
				if (empty($recipient)) {
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'mail', 'hmt_error_missing_reasonrequest_recipient', Hpls_Consts0::ERR_MAIL_NO_RECIPIENT);
					return;
				}
				$subject = 'Mailtrace Reason Request';
				$sender = $cfg->smtp->sender_address;
				if (empty($sender)) {
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'mail', 'hmt_error_missing_smtp_sender', Hpls_Consts0::ERR_MAIL_NO_SENDER);
					return;
				}
				$this->log()->debug(Hpls_Debug::print_r($sender, __METHOD__));
				$msgbody = '';
				$msgbody .= 'Erkannter Status: HPLS>>>' . $deliver->getStatus() . "<<<HPLS\n";
				$msgbody .= 'Absendender Host: HPLS>>>' . $connect->getClient() . "<<<HPLS\n";
				$msgbody .= 'Empfangende Host: HPLS>>>' . $deliver->getRelayHost() . "<<<HPLS\n";
				$msgbody .= 'Gemeldeter Übertragungsstatus: HPLS>>>' . $deliver->getOriginalReason() . "<<<HPLS\n";
				
				$this->log()->debug(Hpls_Debug::print_r($msgbody, __METHOD__));
				try {
					require_once 'Hpls/Utils/Mail.php';
					Hpls_Utils_Mail::sendMail($recipient, $subject, $msgbody, $sender);
					$this->view->send_success = true;
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_INFO, 'mail', 'hmt_reason_request_send');
				} catch (Exception $ex) {
					$this->view->send_success = false;
					Hpls_Msgs::i()->addGlobalMsg(Hpls_Msgs::MSG_ERROR, 'mail', $ex->getMessage(), $ex->getCode());
				}
			} else {
				Hpls_Msgs::i()->addFieldError('must_accept', 'hmt_accept_send', 'hmt_reason_request_error_must_accept', -1);
			
			}
		}
	
	}

	protected function loadEntries() {
		$searchArray = array();
		$did = null;
		$cid = null;
		$part = null;
		if ($this->getRequest()->has('cid')) {
			$cid = $this->getRequest()->getParam('cid');
		}
		if ($this->getRequest()->has('did')) {
			$split = explode('d', $this->getRequest()->getParam('did'));
			if (count($split) == 1) {
				$did = $split[0];
			} elseif (count($split) == 2) {
				$did = $split[1];
				$part = $split[0];
			}
		
		}
		
		if ($this->getRequest()->has('rpart')) {
			$part = $this->getRequest()->getParam('rpart');
		}
		
		//no/incorrect params given?
		if (is_null($did) || is_null($part)) {
			$this->_redirect("/");
			return;
		}
		
		//@todo ab hier neu
		//get entry
		$clause = new Hpls_Var_Clause();
		$clause->addValue(Hpls_Mailtrace_Consts::SRCH_D_ID, $did);
		
		$qry = new Hmt_Db_Query_Delivers($clause);
		$qry->setPartition($part);
		Hmt_Log::i()->connect($qry);
		$qry->execute();
		$result = $qry->getResults();
		if (count($result) == 0) {
			$this->_forward('error');
			return;
		}
		
		$this->_deliverObj = reset($result);
		isDebug() && $this->getLogger()->debug(Hpls_Debug::print_r($this->_deliverObj, "deliver entry: "));
		
		//@todo werte aus dem connectentry holen
		$cid = $this->_deliverObj->getDBID();
		$clause = new Hpls_Var_Clause();
		$clause->addValue(Hpls_Mailtrace_Consts::SRCH_C_ID, $cid);
		$qry = new Hmt_Db_Query_Connects($clause);
		$qry->setPartition($part);
		Hmt_Log::i()->connect($qry);
		$qry->execute();
		$result = $qry->getResults();
		if (count($result) == 0) {
			$this->_forward('error');
			return;
		}
		
		$this->_connectObj = reset($result);
	
	}

	/**
	 * @todo comment
	 * @return Hmt_Log_DeliverEntry|null
	 */
	protected function getDeliver() {
		if (is_null($this->_deliverObj)) {
			$this->loadEntries();
		
		}
		return $this->_deliverObj;
	}

	/**
	 * @todo comment
	 * @return Hmt_Log_ConnectEntry|null
	 */
	protected function getConnect() {
		if (is_null($this->_connectObj)) {
			$this->loadEntries();
		
		}
		return $this->_connectObj;
	}

}

