<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Aug 14, 2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hmt/Db/Statistics.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hmt_Db_Statistics_Pgsql extends Hmt_Db_Statistics {
	const SQL_TUPLES_COUNT_C = "SELECT relname, reltuples FROM pg_class where (relname ilike 'connectlog_%') ORDER BY relname;";
	const SQL_TUPLES_COUNT_D = "SELECT relname, reltuples FROM pg_class where (relname ilike 'deliverlog_%') ORDER BY relname;";
	const SQL_VACUUM_TIME = "SELECT pg_stat_get_last_vacuum_time((SELECT oid FROM pg_class WHERE relname = '%s'));";
	const SQL_DB_SIZE = "SELECT pg_relation_size('%s');";

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCounts()
	 */
	function getTupleCountsConnect() {
		// TODO Auto-generated method stub
		$db = $this->getDb();
		$stm = $db->query(self::SQL_TUPLES_COUNT_C);
		$ret = 0;
		foreach ($stm->fetchAll(PDO::FETCH_ASSOC) as $r) {
			$matches = array();
			if (preg_match('/^' . Hmt_Db_Consts::TABLE_CONNECT . '(\d+)$/', $r['relname'], $matches) > 0) {
				$ret += floatval($r['reltuples']);
			}
		}
		
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getDbSize()
	 */
	function getDbSize() {
		// TODO Auto-generated method stub
		$ints = Hmt_Log::i()->getIntervals();
		$size = 0;
		try {
			foreach ($ints as $iv) {
				$db = $this->getDb();
				$stm = $db->query(sprintf(self::SQL_DB_SIZE, Hmt_Db_Consts::TABLE_CONNECT . $iv));
				$res = $stm->fetchColumn();
				if (empty($res))
					continue;
				$size += $res;
				$stm = $db->query(sprintf(self::SQL_DB_SIZE, Hmt_Db_Consts::TABLE_DELIVER . $iv));
				$res = $stm->fetchColumn();
				if (empty($res))
					continue;
				$size += $res;
			}
		} catch (Exception $ex) {
			//ein fehler in der DB kommunikation
			Hpls_Utils::log()->err(__METHOD__ . ": $ex");
			return null;
		}
		
		return $size;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getTupleCountsDeliver()
	 */
	function getTupleCountsDeliver() {
		$db = $this->getDb();
		$stm = $db->query(self::SQL_TUPLES_COUNT_D);
		$ret = 0;
		foreach ($stm->fetchAll(PDO::FETCH_ASSOC) as $r) {
			$matches = array();
			if (preg_match('/^' . Hmt_Db_Consts::TABLE_DELIVER . '(\d+)$/', $r['relname'], $matches) > 0) {
				$ret += floatval($r['reltuples']);
			}
		}
		
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hmt_Db_Statistics::getLastReorg()
	 */
	function getLastReorg() {
		// TODO Auto-generated method stub
		$ints = Hmt_Log::i()->getIntervals();
		$time = 0;
		try {
			foreach ($ints as $iv) {
				$db = $this->getDb();
				$stm = $db->query(sprintf(self::SQL_VACUUM_TIME, Hmt_Db_Consts::TABLE_CONNECT . $iv));
				$time = $stm->fetchColumn();
				if (!empty($time))
					break;
			}
		} catch (Exception $ex) {
			//ein fehler in der DB kommunikation
			Hpls_Utils::log()->err(__METHOD__ . ": $ex");
			return null;
		}
		
		return $time;
	}

}