<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 08.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-04-15 16:36:08 +0200 (So, 15. Apr 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9311 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hmt/Reason/Edit.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/**
 */
require_once 'Hmt/Reason/Manage.php';
require_once 'Hmt/Reason/Edit/Rule.php';
require_once 'Hmt/Reason/Edit/RegEx.php';
require_once 'Hmt/Reason/Edit/Text.php';

/**
 *
 * @todo comment
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9311 $
 * @since 1.0.9
 */
class Hmt_Reason_Edit extends Hmt_Reason_Manage {
	protected static $_instance2 = null;

	/**
	 *
	 * @return Hmt_Reason_Edit
	 */
	public static function i() {
		if (is_null(self::$_instance2)) {
			self::$_instance2 = new self();
		}
		return self::$_instance2;
	}
	
	protected $_currentTarget = 'customer';

	protected function __construct() {
		parent::__construct();
	}

	public function getRules() {
		$ret = parent::getRules();
		return $ret;
	}

	public function getRoles() {
		$ret = parent::getRoles();
		return $ret;
	}

	public function getReasonTextById($lang, $textid) {
		
		$ret = parent::getReasonTextById($lang, $textid);
		return $ret;
	}

	public function getRegExs($ruleID) {
		$ret = parent::getRegExs($ruleID);
		foreach ($ret as $regex) {
			if (!isset($regex->fromtable)) {
				$regex->fromtable = $this->getCurrentTarget();
			}
		}
		return $ret;
	}
	
	//	public function getReasonText($ruleID, $lang = null, $role = null) {
	//		$ret = parent::getReasonText($ruleID, $lang, $role);
	//		foreach ($ret as $regex) {
	//			
	//			if (is_object($regex) && !isset($regex->fromtable)) {
	//				$regex->fromtable = $this->getCurrentTarget();
	//			}
	//		}
	//		return $ret;
	//	}
	

	/* (non-PHPdoc)
	 * @see Hmt_Reason::fetch()
	 */
	public function fetch($stm, $inFunction) {
		$fetchClass = null;
		switch ($inFunction) {
			case 'getRules' :
			case 'getRule' :
				$fetchClass = 'Hmt_Reason_Edit_Rule';
				break;
			case 'getReasonText' :
				$fetchClass = 'Hmt_Reason_Edit_Text';
				break;
			
			case 'getRegExs' :
			case 'getRegEx' :
				$fetchClass = 'Hmt_Reason_Edit_RegEx';
		}
		if (!empty($fetchClass)) {
			$ret = array();
			while ($row = $stm->fetchObject($fetchClass)) {
				//				$row = array_change_key_case($row, CASE_LOWER);
				//				$row->setFromtable($this->getCurrentTarget());
				$ret[] = $row;
			}
			return $ret;
		} else {
			return parent::fetch($stm, $inFunction);
		}
	
	}

	public function setTargetDb($name) {
	
	}

	public function saveRule(Hmt_Reason_Edit_Rule $rule) {
		$values = array();
		$values['inactive_state'] = 0;
		if ($rule->getName()) {
			$values['rule_name'] = $rule->getName();
		}
		if ($rule->getDescription()) {
			$values['description'] = $rule->getDescription();
		}
		if ($rule->isDeactivated()) {
			$values['inactive_state'] = 1;
		}
		if ($rule->hasID()) {
			$values['id'] = $rule->getIdS();
			$values['id_t'] = $rule->getIdT();
		}
		//		$this->log()->debug(Hpls_Debug::print_r($rule, __METHOD__));
		$this->log()->debug(Hpls_Debug::print_r($values, __METHOD__));
		
		$this->log()->debug(__METHOD__ . ' saving to ' . $this->getCurrentTarget());
		
		$db = $this->getDb($this->getCurrentTarget());
		try {
			//INSERT
			$this->log()->debug(Hpls_Debug::print_r($db, __METHOD__));
			$in = $db->insert(self::TABLE_RULES, $values);
			
			$rule->setIdS($db->lastInsertId(self::TABLE_RULES, 'id'));
			return $in;
		} catch (Exception $ex) {
			$this->log()->info(__METHOD__ . " $ex");
			if ($rule->hasID()) {
				//UPDATE
				$this->log()->debug(__METHOD__ . " UPDATE");
				return $db->update(self::TABLE_RULES, $values, array(
						'id = ?' => $rule->getIdS(),
						'id_t = ?' => $rule->getIdT() ));
			} else {
				require_once 'Hpls/Mailtrace/Exception/Reason/Editor.php';
				throw new Hpls_Mailtrace_Exception_Reason_Editor(__METHOD__);
			}
		}
	}

	public function saveText(Hmt_Reason_Edit_Text $text) {
		$lang = $text->getLanguage();
		if (!in_array($lang, $this->getAvaiableLanguages())) {
			require_once 'Hpls/Mailtrace/Exception/Reason.php';
			throw new Hpls_Mailtrace_Exception_Reason("Language '$lang' is not avaiable.");
		
		}
		$values = array();
		$values['tag_desc'] = null;
		$values['short_desc'] = null;
		$values['long_desc'] = null;
		
		if ($text->getTextTag()) {
			$values['tag_desc'] = $text->getTextTag();
		}
		if ($text->getTextLong()) {
			$values['long_desc'] = $text->getTextLong();
		}
		if ($text->getTextShort()) {
			$values['short_desc'] = $text->getTextShort();
		}
		if ($text->getRuleId()) {
			$values['rule_id'] = $text->getRuleIdS();
			$values['rule_id_t'] = $text->getRuleIdT();
		}
		if ($text->getRoleId()) {
			$values['role_id'] = $text->getRoleIdS();
			$values['role_id_t'] = $text->getRoleIdT();
		}
		if ($text->hasID()) {
			$values['id'] = $text->getIdS();
			$values['id_t'] = $text->getIdT();
		}
		
		$this->log()->debug(Hpls_Debug::print_r($text, __METHOD__));
		$this->log()->debug(Hpls_Debug::print_r($values, __METHOD__));
		
		$this->log()->debug(__METHOD__ . ' saving to ' . $this->getCurrentTarget());
		
		try {
			$in = $this->getDb()->insert(self::TABLE_REASON_PREFIX . $lang, $values);
			$text->setIdS($this->getDb()->lastInsertId(self::TABLE_REASON_PREFIX . $lang, 'id'));
			//@todo implement set ID_T
			return $in;
		} catch (Exception $ex) {
			$this->log()->info(__METHOD__ . " $ex");
			if ($text->hasID()) {
				$this->log()->debug(__METHOD__ . " UPDATE");
				return $this->getDb()->update(self::TABLE_REASON_PREFIX . $lang, $values, array(
						'id = ?' => $text->getIdS(),
						'id_t = ?' => $text->getIdT() ));
			} else {
				require_once 'Hpls/Mailtrace/Exception/Reason/Editor.php';
				throw new Hpls_Mailtrace_Exception_Reason_Editor(__METHOD__);
			}
		}
	}

	public function saveRegEx(Hmt_Reason_Edit_RegEx $rule) {
		$values = array();
		if (!is_null($rule->getRegEx())) {
			$values['regex'] = $rule->getRegEx();
		}
		$map = trim($rule->getParamMap());
		if (!is_null($map)) {
			$values['param_map'] = $rule->getParamMap();
		}
		if (!is_null($rule->getPriority())) {
			$values['priority'] = $rule->getPriority();
		}
		if ($rule->getRuleId()) {
			$values['rule_id'] = $rule->getRuleIdS();
			$values['rule_id_t'] = $rule->getRuleIdT();
		}
		$values['inactive_state'] = $rule->isDeactivated() ? 1 : 0;
		if ($rule->getDescription()) {
			$values['description'] = $rule->getDescription();
		}
		if ($rule->hasID()) {
			$values['id'] = $rule->getIdS();
			$values['id_t'] = $rule->getIdT();
		}
		
		$this->log()->debug(Hpls_Debug::print_r($rule, __METHOD__));
		$this->log()->debug(Hpls_Debug::print_r($values, __METHOD__));
		
		$this->log()->debug(__METHOD__ . ' saving to ' . $this->getCurrentTarget());
		try {
			//INSERT
			$in = $this->getDb()->insert(self::TABLE_REGEX, $values);
			$rule->setIdS($this->getDb()->lastInsertId(self::TABLE_REGEX, 'id'));
			return $in;
		} catch (Exception $ex) {
			$this->log()->info(__METHOD__ . " $ex");
			if ($rule->hasID()) {
				//UPDATE
				$this->log()->debug(__METHOD__ . " UPDATE");
				return $this->getDb()->update(self::TABLE_REGEX, $values, array(
						'id = ?' => $rule->getIdS(),
						'id_t = ?' => $rule->getIdT() ));
			} else {
				require_once 'Hpls/Mailtrace/Exception/Reason/Editor.php';
				throw new Hpls_Mailtrace_Exception_Reason_Editor(__METHOD__);
			
			}
		} //catch
	}

	/**
	 *
	 * @todo comment
	 * @param $toDelId String
	 * @param $target String
	 */
	public function delRule($toDelId, $target = null) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		
		if (is_null($target)) {
			$target = $this->getCurrentTarget();
		}
		
		$this->log()->debug(__METHOD__ . ' change inactive_state with id (' . $toDelId . ') on ' . $target);
		
		$db = $this->getDb($target);
		$this->log()->debug(Hpls_Debug::print_r($db, __METHOD__ . ' SNi2'));
		$db->update(self::TABLE_RULES, array(
				Hmt_Reason_Db::FIELD_STATE => self::INACTIVESTATE_DELETED ), array(
				'id = ?' => $id[0],
				'id_t = ?' => $id[1] ));
		
		Hmt_Reason_Edit::i()->clearCache();
	}

	/**
	 *
	 * @todo comment
	 * @param $toDelId String
	 * @param $target String
	 */
	public function delRegEx($toDelId, $target = null) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		
		if (is_null($target)) {
			$target = $this->getCurrentTarget();
		}
		
		$db = $this->getDb($target);
		isDebug('hmt_reason_edit') && $this->log()->debug(__METHOD__ . " Setting inactive state of regex '$toDelId' to deleted.");
		$resCount = $db->update(self::TABLE_REGEX, array(
				Hmt_Reason_Db::FIELD_STATE => self::INACTIVESTATE_DELETED ), array(
				'id = ?' => $id[0],
				'id_t = ?' => $id[1] ));
		if ($resCount == 0) {
			$this->log()->err(__METHOD__ . " Could not set deleted on regex '$toDelId'");
		}
		
		Hmt_Reason_Edit::i()->clearCache();
	}

	/**
	 *
	 * @todo comment
	 * @param $id unknown_type
	 */
	public function delRText($lang, $toDelId) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		if (in_array($lang, $this->getAvaiableLanguages()))
			$this->getDb()->update(self::TABLE_REASON_PREFIX . $lang, array(
					Hmt_Reason_Db::FIELD_STATE => self::INACTIVESTATE_DELETED ), array(
					'id = ?' => $id[0],
					'id_t = ?' => $id[1] ));
	}

	/**
	 * Löscht eine Regel und alle von ihr abhängigen Einträge aus der Datenbank.
	 * 
	 * @param $toDelId integer
	 */
	public function realDeleteRule($toDelId) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		foreach ($this->getAvaiableLanguages() as $lang) {
			
			$this->getDb()->delete(self::TABLE_REASON_PREFIX . $lang, array(
					'rule_id = ?' => $id[0],
					'rule_id_t = ?' => $id[1] ));
		}
		$this->getDb()->delete(self::TABLE_REGEX, array(
				'rule_id = ?' => $id[0],
				'rule_id_t = ?' => $id[1] ));
		$this->getDb()->delete(self::TABLE_RULES, array(
				'id = ?' => $id[0],
				'id_t = ?' => $id[1] ));
	}

	/**
	 *
	 * @todo comment
	 * @param $toDelId integer
	 */
	public function realDeleteRegEx($toDelId) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		$this->getDb()->delete(self::TABLE_REGEX, array(
				'id = ?' => $id[0],
				'id_t = ?' => $id[1] ));
	}

	/**
	 *
	 * @todo comment
	 * @param $lang string
	 * @param $toDelId integer
	 */
	public function realDeleteText($lang, $toDelId) {
		$id = explode(self::ID_SEP, $toDelId);
		if (count($id) != 2) {
			throw new InvalidArgumentException("Argument with '$toDelId' is not in a format of '<id-seq-nr>-<id_t>'", -1);
		}
		if (in_array($lang, $this->getAvaiableLanguages()))
			$this->getDb()->delete(self::TABLE_REASON_PREFIX . $lang, array(
					'id = ?' => $id[0],
					'id_t = ?' => $id[1] ));
	}

	/**
	 * (non-PHPdoc)
	 * 
	 * @return Hmt_Reason_Db
	 */
	public function getDbO($name = null) {
		if (is_null($name))
			$name = $this->_currentTarget;
		
		return parent::getDbO($name);
	}

	/**
	 *
	 * @return the $_currentTarget
	 */
	public function getCurrentTarget() {
		return $this->_currentTarget;
	}

	/**
	 *
	 * @param $_currentTarget field_type
	 */
	public function setCurrentTarget($_currentTarget) {
		$this->_currentTarget = $_currentTarget;
	}

	protected function getIdTFromInsert($table, $lastInsertId) {
	
	}

}