<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : May 19, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2012-01-02 14:48:43 +0100 (Mo, 02. Jan 2012) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9197 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Acl/Reader/Abstract.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Consts.php';
require_once 'Hpls/Acl/Consts.php';
require_once 'Zend/Acl.php';
require_once 'Hpls/Acl/Reader/Interface.php';
require_once 'Hpls/Acl/Ids.php';
require_once 'Hpls/Acl/Ops.php';
require_once 'Hpls/Acl/Resource.php';
require_once 'Hpls/Acl/Role.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AppCore
 * @subpackage Rechtesystem
 * @version $LastChangedRevision: 9197 $
 * @since rc
 */
abstract class Hpls_Acl_Reader_Abstract implements 
	Hpls_Acl_Reader_Interface {
	protected $_configPath;
	/**
	 * @var Hpls_Acl_ZendExtend
	 */
	protected $_ACL = null;
	protected $_source = null;

	protected function __construct($configPath = Hpls_Consts::APP_CONFIG_PATH) {
		$this->_configPath = realpath($configPath);
		if (!file_exists($this->_configPath)) {
			require_once 'Hpls/Exception/Acl/Reader.php';
			throw new Hpls_Exception_Acl_Reader("Configuration directory '$this->_configPath' does not exists.");
		}
	}

	public function setTarget(Zend_Acl $ACL) {
		$this->_ACL = $ACL;
	}

	public function setSource($source) {
		$this->_source = $source;
	}

	public function loadResources(Hpls_Acl $ACL) {
		$resources = $this->readRawResource();
		$ACL->setModuleMapping($this->readModuleResMap());
		$this->processResourceObj($resources, null);
	}

	public function loadRoles(Hpls_Acl $ACL) {
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles GUEST...');
		$parent = new Hpls_Acl_Role(Hpls_Acl_Role::GUEST);
		$this->_ACL->addRole($parent);
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles LOGGEDIN...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::LOGGEDIN);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles USER...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::USER);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles MAILUSER...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::MAILUSER);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles POSTMASTER...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::POSTMASTER);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading roles SUPERPOSTMASTER...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::SUPER_POSTMASTER);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading role HELPDESK...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::HELPDESK);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading role ADMIN...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::ADMIN);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading role SUPERADMIN...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::SUPER_ADMIN);
		$this->_ACL->addRole($next, $parent);
		$parent = $next;
		isDebug('acl_read') && $this->log()->debug(__METHOD__.' loading role HPLSADMIN...');
		$next = new Hpls_Acl_Role(Hpls_Acl_Role::HPLS_ADMIN);
		$this->_ACL->addRole($next, $parent);
	}

	public function loadRules() {
	
	}

	/**
	 * Die eigentliche Verarbeitung der rohen ACL-Daten und aufarbeitung zum Zend_Acl-nutzbaren Regelwerk.
	 * @param array $resources
	 * @param Hpls_Acl_Resource $parentRes
	 */
	protected function processResourceObj(array $resources, Hpls_Acl_Resource $parentRes = null) {
		if (is_null($parentRes)) {
			//root of ACL
			$parentKey = "";
			$parentRes = $this->_ACL->getRootResource();
		} else {
			$parentKey = $parentRes->getResourceId() . Hpls_Acl_Consts::ID_SEPARATOR;
		}
		
		foreach ($resources as $key => $res) {
			if (strcmp($key, Hpls_Acl_Consts::ACL_SECTION_PRIVILEGES) === 0) {
				foreach ($res as $privName => $privValue) {
					$parentRes->setValue($privName, $privValue);
				}
			
		//				Hpls_Utils::log()->debug("Updated ACL-Res: $parentRes");
			} else {
				$resObj = new Hpls_Acl_Resource($parentKey . $key);
				$this->_ACL->add($resObj, $parentRes);
				//			Hpls_Utils::log()->debug("Created ACL-Res: $resObj");
				

				if (is_array($res)) {
					$this->processResourceObj($res, $resObj);
				}
			}
		}
	
	}

	/**
	 * @return array
	 */
	protected abstract function readRawResource();

	/**
	 * @todo comment
	 * [<module_name>] {
	 * [__resources__] {
	 * }
	 * [<controller-name>] {
	 * [__resources__] {
	 * }
	 * [<action-name>] {
	 * [__resources__] {
	 * }
	 * [<param-name>] {
	 * }
	 * } 
	 * }
	 * } 
	 * @return Zend_Config
	 */
	protected abstract function readModuleResMap();

	protected function getACL() {
		return $this->_ACL;
	}

	protected function getAclDirs($startDir = null) {
		if (is_null($startDir)) {
			$startDir = $this->_configPath;
		}
		$ret = array();
		$files = @scandir($startDir); //unterdrückt wg. datei-rechten
		foreach ($files as $file) {
			isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . ": Scanning $startDir/$file");
			$filePath = "$startDir/$file";
			if (is_dir($filePath) && strcmp($file, '.') !== 0 && strcmp($file, '..') !== 0) {
				if ($this->containsAclDir($filePath)) {
					isDebug('acl', 'acl_read') && $this->log()->debug(__METHOD__ . ": found '$filePath'");
					$ret[] = "$filePath/" . Hpls_Acl_Consts::APP_ACL_DIR;
				}
			}
		}
		return $ret;
	}

	protected function containsAclDir($dir) {
		if (!is_readable($dir)) {
			return false;
		}
		$files = scandir($dir);
		if (is_array($files)) {
			foreach ($files as $file) {
				//			Hpls_Utils::log()->debug("Checking $dir/$file");
				if (is_dir("$dir/$file") && strcmp($file, Hpls_Acl_Consts::APP_ACL_DIR) === 0) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @todo Zend_Log
	 */
	protected function log() {
		return Hpls_Utils::log();
	}

}