<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : Apr 20, 2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Application/Config/Outputtype.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Application/ConfigAbstract.php';

/**
 * @todo comment
 * 
 * Bestimmt den verlangten Outputtype des Request.
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage subpackage
 * @version $LastChangedRevision: 9196 $
 * @since alpha.1
 *
 */
class Hpls_Application_Config_Outputtype extends Hpls_Application_ConfigAbstract {
	/**
	 * Enter description here ...
	 * @var Zend_Uri_Http
	 */
	protected $_uriOutput = null;
	protected $_acceptedMime = null;

	public function init() {
		isDebug('config_plugins_init', 'method_trace') && $this->log()->info(__METHOD__ . ' - INIT');
		$front = $this->getFrontController();
		$req = new Zend_Controller_Request_Http();
		
		$params = $front->getRouter()->route($req);
		if ($params !== false) {
			$this->_uriOutput = $params->getParam('output', null);
			if ($this->_uriOutput !== null) {
				$this->_uriOutput = strtolower($this->_uriOutput);
			}
		}
		
		isDebug('config_plugins_init') && $this->log()->debug(Hpls_Debug::print_r($params, __METHOD__));
		
		$url = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
		
		// Erwartet diese Anfrage HTML, XML oder PNG?
		$lastEl = basename($url);
		$ctype = '';
		if ($this->isXmlRequested() || strcasecmp($lastEl, 'xml') == 0) {
			isDebug('config_plugins_init') && $this->log()->debug("Outputtype XML");
			$ctype = Hpls_Consts::FRONTP_OUTTYPE_XML;
		} elseif ($this->isJsonRequested() || strcasecmp($lastEl, 'json') == 0) {
			
			isDebug('config_plugins_init') && $this->log()->debug("Outputtype JSON");
			$ctype = Hpls_Consts::FRONTP_OUTTYPE_JSON;
		} elseif (strcasecmp($lastEl, 'png') == 0) {
			$ctype = Hpls_Consts::FRONTP_OUTTYPE_PNG;
		} elseif (strcasecmp($lastEl, 'pdf') == 0) {
			$ctype = Hpls_Consts::FRONTP_OUTTYPE_PDF;
		} else {
			$ctype = Hpls_Consts::FRONTP_OUTTYPE_HTML;
		}
		$front->setParam(Hpls_Consts::FRONTP_OUTTYPE, $ctype);
		isDebug('config_plugins_init') && $this->log()->debug(__METHOD__ . ' - INIT END');
	}

	protected function isJsonRequested() {
		//		[HTTP_ACCEPT] => application/json, text/javascript, */*; q=0.01
		//		$this->log()->info(__METHOD__ . ' - Json');
		//		if (($this->_uriOutput === 'json') || $this->acceptedMimeType('application/json') || $this->acceptedMimeType('text/javascript')) {
		//			return true;
		//		}
		return false;
	}

	protected function isXmlRequested() {
		if (($this->_uriOutput === 'xml') || $this->acceptedMimeType('application/xml')) {
			return true;
		}
		return false;
	}

	protected function isXmlHttpRequest() {
		isDebug('config_plugins_init') && $this->log()->info(Hpls_Debug::print_r($_SERVER, "SERVER"));
		
		return (strcasecmp($this->getHeader('X_REQUESTED_WITH'), 'XMLHttpRequest') == 0);
	}

	/**
	 * Return the value of the given HTTP header. Pass the header name as the
	 * plain, HTTP-specified header name. Ex.: Ask for 'Accept' to get the
	 * Accept header, 'Accept-Encoding' to get the Accept-Encoding header.
	 *
	 * @param string $header HTTP header name
	 * @return string|false HTTP header value, or false if not found
	 * @throws Zend_Controller_Request_Exception
	 */
	public function getHeader($header) {
		if (empty($header)) {
			require_once 'Zend/Controller/Request/Exception.php';
			throw new Zend_Controller_Request_Exception('An HTTP header name is required');
		}
		
		// Try to get it from the $_SERVER array first
		$temp = 'HTTP_' . strtoupper(str_replace('-', '_', $header));
		if (isset($_SERVER[$temp])) {
			return $_SERVER[$temp];
		}
		
		// This seems to be the only way to get the Authorization header on
		// Apache
		if (function_exists('apache_request_headers')) {
			$headers = apache_request_headers();
			if (isset($headers[$header])) {
				return $headers[$header];
			}
			$header = strtolower($header);
			foreach ($headers as $key => $value) {
				if (strtolower($key) == $header) {
					return $value;
				}
			}
		}
		
		return false;
	}

	protected function acceptedMimeType($mtype) {
		if (is_null($this->_acceptedMime)) {
			if (isset($_SERVER['HTTP_ACCEPT'])) {
				//		[HTTP_ACCEPT] => application/json, text/javascript, */*; q=0.01
				$mime = explode(',', $_SERVER['HTTP_ACCEPT']);
				array_walk($mime, 'array_walk_trim');
				$this->_acceptedMime = $mime;
			} else {
				$this->_acceptedMime = false;
			}
		}
		if (is_array($this->_acceptedMime)) {
			return in_array($mtype, $this->_acceptedMime);
		}
		
		return false;
	}

}
