<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 27.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-08-14 21:14:28 +0200 (Sun, 14 Aug 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 8635 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/branches/for_20110120_performance/hpls.ac.core2.gui.common/Hpls/User/Persistence/Ini.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Auth/Reader/UserdataAbstract.php';
require_once 'Hpls/Storage/ReaderAbstract.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 8635 $
 * @since 1.0.9
 */
class Hpls_Auth_Reader_Userdata_Ini extends Hpls_Auth_Reader_UserdataAbstract {
	const OPT_FILE = 'file';
	const OPT_INI_EMAIL_KEY = 'emailkey';
	const OPT_INI_USERNAME_KEY = 'usernamekey';
	const FIELD_EMAIL = 'email';
	const FIELD_DOMAIN = 'domain';
	protected $_ini = null;

	/**
	 * @todo comment
	 * @return NULL
	 */
	protected function getZendConfig() {
		$this->log()->debug(Hpls_Debug::print_r($this->_options, __METHOD__ . " - options "));
		if (is_null($this->_ini)) {
			$filename = trim($this->_options['file']);
			$posFirstSlash = stripos($filename, DIRECTORY_SEPARATOR);
			if (false === $posFirstSlash) {
				$this->log()->debug(Hpls_Debug::print_r($filename, __METHOD__ . " - ini cascaded "));
				$this->_ini = Hpls_Utils::readConfigIniCascade($filename);
			} else if ($posFirstSlash === 0) {
				//absolute path
				$this->log()->debug(Hpls_Debug::print_r($filename, __METHOD__ . " - ini absolute "));
				if (file_exists($filename)) {
					$this->_ini = new Zend_Config_Ini($filename);
				} else {
					require_once 'Hpls/Exception/Configuration.php';
					throw new Hpls_Exception_Configuration("Specified file '$filename' in auth.xml/[{$this->_name}] does not exist.");
				}
			} else {
				//relative path to config dir
				$filename = Hpls_Application::i()->getConfigDir() . DIRECTORY_SEPARATOR . $filename;
				$this->log()->debug(Hpls_Debug::print_r($filename, __METHOD__ . " - ini relative "));
				if (file_exists($filename)) {
					$this->_ini = new Zend_Config_Ini($filename);
				} else {
					require_once 'Hpls/Exception/Configuration.php';
					throw new Hpls_Exception_Configuration("Specified file '$filename' in auth.xml/[{$this->_name}] does not exist.");
				}
			}
		}
		return $this->_ini;
	}

	/* (non-PHPdoc)
	 * @see Hpls_Auth_Reader_UserdataAbstract::checkOptions()
	 */
	public function checkOptions() {
		// TODO Auto-generated method stub
		if (!is_array($this->_options)) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('Options are not an array in user settings storage "' . $this->_name . '".', -1);
		}
		if (!isset($this->_options[self::OPT_FILE])) {
			require_once 'Hpls/Exception/Configuration.php';
			throw new Hpls_Exception_Configuration('Option "file" is missing for user settings storage "' . $this->_name . '".', -2);
		}
		//		if (!file_exists($this->_options[self::OPT_FILE])) {
		//			require_once 'Hpls/Exception/Configuration.php';
		//			throw new Hpls_Exception_Configuration('The file "' . $this->_options[self::OPT_FILE] . '" does not exists in user settings storage "' . $this->_name . '".', -2);
		//		}
		

		if (empty($this->_options[self::OPT_INI_EMAIL_KEY])) {
			$this->_options[self::OPT_INI_EMAIL_KEY] = '[[' . Hpls_Consts0::AUTH_VAR_LOGIN . ']]/emails';
		}
	}

	/* (non-PHPdoc)
	 * @see Hpls_Storage_ReaderInterface::connect()
	 */
	public function connect() {
		// TODO Auto-generated method stub
	

	}

	/* (non-PHPdoc)
	 * @see Hpls_Storage_ReaderInterface::disconnect()
	 */
	public function disconnect() {
		// TODO Auto-generated method stub
	

	}

	/* (non-PHPdoc)
	 * @see Hpls_Storage_ReaderInterface::getValues()
	 */
	public function getValues() {
		// TODO Auto-generated method stub
	

	}

	/* (non-PHPdoc)
	 * @see Hpls_Auth_Reader_UserdataInterface::getManagedDomains()
	 */
	public function getManagedDomains($username = null) {
		$ret = array();
		$ini = $this->getZendConfig();
		if (isset($ini->{$username})) {
			$userIni = $ini->{$username};
			if (isset($userIni->{self::FIELD_DOMAIN})) {
				foreach ($userIni->{self::FIELD_DOMAIN} as $email) {
					$ret[] = $email;
				}
			}
		}
		return $ret;
	}

	/* (non-PHPdoc)
	 * @see Hpls_Auth_Reader_UserdataAbstract::getManagedEmails()
	 */
	public function getManagedEmails($username = null) {
		// TODO Auto-generated method stub
		$this->log()->debug(__METHOD__ . "($username) - START");
		$ret = array();
		$this->setUsername($username);
		//		$this->setUsername($username);
		$usernameSubst = $this->getUsernameMapper()->getSubstituted($this->getOption(self::OPT_INI_EMAIL_KEY), $username);
		$this->log()->debug(__METHOD__ . " Reading from '$usernameSubst'");
		$emailEntry = Hpls_Utils::array_deep_value($this->getIniFileContents(), $usernameSubst);
		$this->log()->debug(__METHOD__ . Hpls_Debug::print_r($emailEntry, ' FOUND ENTRIES') . Hpls_Debug::print_r($this->getIniFileContents(), ' IN FILE CONTENTS'));
		if (empty($emailEntry)) {
			return array();
		}
		if (!is_array($emailEntry)) {
			$emailEntry = preg_split('/\s*(,|:|\|)\s*/', $emailEntry);
		}
		$this->log()->debug(Hpls_Debug::print_r($emailEntry, __METHOD__));
		
		return $emailEntry;
	}

	protected function getIniFileContents() {
		$filename = $this->getOption(self::OPT_FILE);
		$cd = stripos($filename, DIRECTORY_SEPARATOR);
		if ($filename instanceof Zend_Config) {
			return $filename->toArray();
		} elseif (is_array($filename)) {
			return $filename;
		} elseif (0 === $cd) {
			$ini = new Zend_Config_Ini($filename);
			return $ini->toArray();
		} else if (false === $cd) { // relativ zu frontend/etc/
			$ini = new Zend_Config_Ini(Hpls_Application0::i()->getVariableConfigDir() . DIRECTORY_SEPARATOR . $filename);
			return $ini->toArray();
		} else {
			//relative file
			$filename = Hpls_Application::i()->getConfigDir() . DIRECTORY_SEPARATOR . $filename;
			$this->log()->debug(Hpls_Debug::print_r($filename, __METHOD__ . " - ini relative "));
			if (file_exists($filename)) {
				return new Zend_Config_Ini($filename);
			} else {
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("Specified file '$filename' in auth.xml/[{$this->_name}] does not exist.");
			}
		}
	}

	/* (non-PHPdoc)
	 * @see Hpls_Auth_Reader_UserdataAbstract::getData()
	 */
	public function getData($username = null) {
		// TODO Auto-generated method stub
	

	}

}