<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 20.08.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Config/Ini.php';
require_once 'Hpls/Utils/Db.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Config_Db extends Zend_Config {
	/**
	 * @todo comment
	 * @var Zend_Db_Adapter_Abstract
	 */
	protected $_db = null;
	protected $_options = null;

	public function __construct($options, $allowModifications = false) {
		$this->setOptions($options);
		$contents = $this->getDbContents();
		$data = array();
		foreach ($contents as $k => $v) {
			$deepData = &$data;
			$deepData = $this->processKey($data, $k, $v);
		}
		parent::__construct($data, $allowModifications);
		
	}

	protected function processKey(array &$data, $k, $v) {
		$pieces = explode($this->getNestSeparator(), $k, 2);
		switch (count($pieces)) {
			case 1 :
				$data[$k] = $this->prepareValue($v);
				break;
			case 2 :
				if (!isset($data[$pieces[0]])) {
					if ($pieces[0] === '0' && !empty($data)) {
						// convert the current values in $config into an array
						$data = array(
								$pieces[0] => $data );
					} else {
						$data[$pieces[0]] = array();
					}
				} elseif (!is_array($data[$pieces[0]])) {
					/**
					 * @see Zend_Config_Exception
					 */
					require_once 'Zend/Config/Exception.php';
					throw new Zend_Config_Exception("Cannot create sub-key for '{$pieces[0]}' as key already exists");
				}
				$data[$pieces[0]] = $this->processKey($data[$pieces[0]], $pieces[1], $v);
				
				break;
		}
		return $data;
	}

	public function setOptions($options) {
		if (!isset($options['nestSeparator'])) {
			$options['nestSeparator'] = '.';
		}
		if (!isset($options['keycol'])) {
			$options['keycol'] = 'key';
		}
		if (!isset($options['valuecol'])) {
			$options['valuecol'] = 'value';
		}
		if (!isset($options['tablename'])) {
			$options['tablename'] = 'config';
		}
		Hpls_Utils::log()->debug(Hpls_Debug::print_r($options,__METHOD__));
		$this->_options = $options;
	}

	/**
	 * Set the nest separator
	 *
	 * @param  string $filename
	 * @return Zend_Config_Writer_Ini
	 */
	public function setNestSeparator($separator) {
		$this->_options['nestSeparator'] = $separator;
		
		return $this;
	}

	public function getNestSeparator() {
		return $this->_options['nestSeparator'];
	}
	
	protected function prepareValue($value) {
		return $value;
	}

	protected function getDbContents() {
		$db = $this->getDb();
		$select = $db->select();
		$select->from(array(
				'c' => $this->getTablename() ), array(
				'k' => $this->_options['keycol'],
				'v' => $this->_options['valuecol'] ));
		$stm = $select->query(PDO::FETCH_ASSOC);
		$res = $stm->fetchAll();
		$ret = array();
		foreach ($res as $r) {
			$ret[$r['k']] = $r['v'];
		}
		return $ret;
	}

	protected function getTablename() {
		return $this->_options['tablename'];
	}

	/**
	 * @todo comment
	 * @return Zend_Db_Adapter_Abstract
	 */
	protected function getDb() {
		if (is_null($this->_db)) {
			$this->_db = Hpls_Utils_Db::mapDriverType2ZendDb($this->_options);
		}
		return $this->_db;
	}

}