<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 12.05.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Constraint/PolicyAbstract.php';

/**
 * Überprüft, ob in den übergebenen Feldern, mindestens eines oder alle eine EMail-Adresse des aktuell angemeldeten User enthalten.
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Constraint_Policy_User_Email extends Hpls_Constraint_PolicyAbstract {
	
	protected $_emails = null;

	/* (non-PHPdoc)
	 * @see Hpls_Constraint_Interface::apply()
	 */
	public function apply($fields, Zend_Controller_Request_Abstract $request) {
		//E-Mail-Adressen des aktuellen Benutzers holen und unifizieren
		if (!is_object(Hpls_Auth::i()->getUserObject())) {
			require_once 'Hpls/Exception/Constraint.php';
			throw new Hpls_Exception_Constraint('No user object.', Hpls_Exception_Constraint::CODE_NO_USER_OBJ);
		}
		$emails = Hpls_Auth::i()->getUserObject()->getEmails();
		$this->_emails = array();
		if (is_array($emails)) {
			foreach ($emails as $email) {
				$this->_emails[] = strtolower($email);
			}
		} else {
			$this->log()->warn(__METHOD__ . " UserObject has invalid email data: " . Hpls_Debug::print_r($emails));
		}
		if (hasRole(Hpls_Acl_Role::POSTMASTER)) {
			return true;
		}
		if (empty($this->_emails)) {
			require_once 'Hpls/Exception/Constraint.php';
			throw new Hpls_Exception_Constraint('User has no E-Mails and at least one is required.');
		}
		// TODO Auto-generated method stub
		$this->_request = $request;
		$this->log()->debug(__METHOD__ . " START " . Hpls_Debug::print_r($fields, 'fields') . Hpls_Debug::print_r($this->_options, 'options'));
		//		Hpls_Msgs::i()->addGlobalMsg('info', __CLASS__, "we started email policy for " . Hpls_Debug::print_r($fields));
		$sess = Hpls_Application::i()->getSession(Hpls_Consts::SESSION_NS_AUTH);
		/**
		 * @var Hpls_User_Object
		 */
		$userObj = $sess->userObj;
		$this->log()->debug(Hpls_Debug::print_r($userObj->getEmails(), __METHOD__ . " EMAILS "));
		//		Hpls_Msgs::i()->addGlobalMsg('info', __CLASS__, '' . Hpls_Debug::print_r($userObj->getEmails()));
		

		if (!is_array($fields)) {
			$fields = array(
					$fields );
		}
		switch ($this->getDepKind()) {
			case self::OPTVAL_DEPENDENCY_ALL :
				$this->applyAllOf($fields, $request);
				break;
			case self::OPTVAL_DEPENDENCY_ONEOF :
				$this->applyOneOf($fields, $request);
				break;
			default :
				require_once 'Hpls/Exception/Configuration.php';
				throw new Hpls_Exception_Configuration("Type of dependency '" . $this->getDepKind() . "' is wrong or not supported as Email-Policy.");
		}
		$this->log()->debug(__METHOD__ . ' END');
	}

	/**
	 * <ul>
	 * <li>Wenn alle beide leer sind, dann in beide Felder die E-Mail-Adressen eintragen und mit 'ODER' verknüpfen
	 * <li>Wenn eins von beiden leer ist, dann dort die e-Mail-Adressen eintragen,
	 * <li>Wenn beide gefüllt sind, überprüfen, ob in EINEM von BEIDEN nur seine E-Mail-Adressen stehen. Ist das nicht
	 * der Fall, könnte er Einblick in andere E-Mails erlangen -> verboten
	 * </ul>
	 * 
	 * @param unknown_type $fields
	 * @param Zend_Controller_Request_Abstract $request
	 */
	protected function applyOneOf(array $fields, Zend_Controller_Request_Abstract $request) {
		$emptyCount = 0;
		$w = array(
				'f' => 0, //foreign email count
				'o' => 0, //own email count
				'empty' => 0,
				'emptyField' => null );
		
		foreach ($fields as $field) {
			switch ($this->containsOnlyAllowed($request->getParam($field, null))) {
				case 0 :
					$w['f']++;
					break;
				case 1 :
					$w['o']++;
					break;
				case 2 :
					if ($w['empty'] === 0) {
						//erstes leeres Feld <- wird später, wenn nötig mit eigenen Mailadressen aufgefüllt.
						$w['emptyField'] = $field;
					}
					$w['empty']++;
					break;
			}
		}
		
		if ($w['f'] === count($fields)) {
			//nur fremde adressen => fehler
			foreach ($fields as $field) {
				$this->addFieldError($field, "hmt_search_limited_to_own_emails__one_have_to_be_only_own");
			}
			return false;
		}
		
		//leere Felder und kein feld mit nur eigenen Mailadressen? -> Handlungsbedarf
		if ($w['empty'] > 0 && $w['o'] === 0) {
			$this->setFieldToOwnEmails($w['emptyField'], $request);
			$this->addFieldMsg(Hpls_Msgs::MSG_WARN, $w['emptyField'], "hmt_search_limited_to_own_emails__one_set_to_your_own__" . $w['emptyField']);
		}
		
		//mindestens 1 nur eigene Mails oder mindestens 1 leeres feld, welches durch eigene mails ersetzt wurde
		

		return true;
	}

	protected function containsOnlyAllowed($fieldValue) {
		if (empty($fieldValue))
			return 2;
		$emails = Hpls_Utils::split_quoted_correct('|', $fieldValue);
		foreach ($emails as $email) {
			if (!$this->isAllowedEmail($email)) {
				return 0;
			}
		}
		return 1;
	}

	protected function isAllowedEmail($email) {
		if (stripos($email, '*') !== false || stripos($email, '?') !== false) {
			//@todo wildcard checken, ob's zu den vorhandenen emails passt
			return false;
		}
		
		return in_array(strtolower($email), $this->_emails);
	}

	protected function setFieldToOwnEmails($field, Zend_Controller_Request_Abstract $request) {
		$request->setParam($field, implode('|', $this->_emails));
	}

	/**
	 * 
	 * @todo comment
	 * @param unknown_type $fields
	 * @param Zend_Controller_Request_Abstract $request
	 * @deprecated match eigentlich hier nicht wirklich sinn
	 */
	protected function applyAllOf($fields, Zend_Controller_Request_Abstract $request) {
		//oneOf und allOf werden hier gleich gestellt, da kein unterschied
		$this->applyOneOf($fields, $request);
	}
}