<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : 25.08.2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Executable/Service.php $
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Executable/Wrapper.php';
require_once 'Hpls/Utils/Logusing.php';

/**
 * @todo comment
 *
 * @author author
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since Magdeburg:Produktiv1
 */
abstract class Hpls_Executable_Service extends Hpls_Utils_Logusing {
	
	protected $_serviceName = null;
	protected $_stopByOneError = true;

	/**
	 * @param string $serviceName Der Name des Service (Skriptname in /etc/init.d)
	 */
	public function __construct($serviceName) {
		$this->DEBUG = TRUE;
		$this->setService($serviceName);
	}

	/**
	 * @param string $serviceName
	 */
	public function setService($serviceName) {
		$this->_serviceName = $serviceName;
	}

	/**
	 * @todo comment
	 *
	 * @return string|NULL
	 */
	public function getService() {
		return $this->_serviceName;
	}

	/**
	 * Existiert der Service auf dem Host?
	 * @return boolean
	 */
	abstract public function exists($host = null);

	/**
	 * @param string $host
	 * @return boolean true: success (Return code == 0), false: any error occured (Return code != 0)
	 */
	abstract public function status($host = null);

	/**
	 * @param string $host
	 * @return boolean true: success (Return code == 0), false: any error occured (Return code != 0)
	 */
	abstract public function start($host = null);

	/**
	 * @param string $host
	 * @return boolean true: success (Return code == 0), false: any error occured (Return code != 0)
	 */
	abstract public function stop($host = null);

	/**
	 * @param string $host
	 * @return boolean true: success (Return code == 0), false: any error occured (Return code != 0)
	 */
	abstract public function reload($host = null);

	/**
	 * Führt den Restart eines Dienstes durch.
	 * @param string|NULL $host
	 * @return boolean <true>, wenn das Programm einen return-code von 0 geliefert hat. Für den eigentlichen Returncode getReturnCode() befragen.
	 */
	abstract public function restart($host = null);

	public function isServiceRunning($host = null) {
		return $this->status($host);
	}

	/**
	 * @param unknown_type $value
	 */
	public function setStopByOneError($value = true) {
		$this->_stopByOneError = $value;
	}

	/**
	 * @todo comment
	 *
	 * @return string
	 */
	public function isStopByOneError() {
		return $this->_stopByOneError;
	}

	/**
	 * @param string|NULL $host Der Hostname/IP, auf dem der Service manipuliert wurde, NULL => LOCALHOST
	 */
	abstract public function getOutput($host = null);

	/**
	 * @param string|NULL $host Der Hostname/IP, auf dem der Service manipuliert wurde, NULL => LOCALHOST
	 * @return int|NULL Der Returncode. NULL, wenn der Service noch nicht mit (re)start/reload/stop/status gerufen wurde.
	 */
	abstract public function getReturnCode($host = null);

	/**
	 * @param unknown_type $hosts
	 */
	abstract public function setHosts($hosts);

	/**
	 *
	 */
	abstract public function getHosts();

	/**
	 * @param unknown_type $user
	 */
	abstract public function setRemoteUser($user = null);

	/**
	 *
	 */
	abstract public function getRemoteUser();
}