<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created        : 01.10.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Executable/Wrapper/Crm.php $
 * 
 * author         : author
 */
/***/
require_once 'Hpls/Executable/WrapperAbstract.php';
require_once 'Hpls/Debug.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage HA
 * @version $LastChangedRevision: 9196 $
 * @since Magdeburg:Produktiv1
 */
class Hpls_Executable_Wrapper_Crm extends Hpls_Executable_WrapperAbstract {
	const CRM_EXECUTABLE = '/usr/sbin/crm';
	const CRM_RESOURCE_EXECUTABLE = '/usr/sbin/crm_resource';
	const CRM_MON_EXECUTABLE = '/usr/sbin/crm_mon';
	
	protected $_configBackup = null;

	public function __construct() {
		parent::__construct(self::CRM_EXECUTABLE);
		$this->setExecWithSudo();
	}

	public function backupConfig() {
		$this->_configBackup = $this->getCompleteConfig();
		return $this->_configBackup;
	}

	public function restoreConfig($crmConfig = null) {
		if (is_null($crmConfig)) {
			$crmConfig = $this->_configBackup;
		}
		if (is_null($crmConfig)) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm('Keine Backup-Daten zum wiederherstellen vorhanden.', 1, null);
		}
		$tmpfile = $this->getTmpDir() . '/' . uniqid(__CLASS__);
		if (FALSE === file_put_contents($tmpfile, $crmConfig)) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm("Konnte '$tmpfile' nicht schreiben.", 1, null);
		}
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'configure',
				'load',
				"\"$tmpfile\"" ));
		if (FALSE === $this->exec()) {
			unlink($tmpfile);
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm("Restore der CRM aus '$tmpfile' ist gescheitert.", $this->getReturnCode(), $this->getCommand(), 'localhost', $this->getCommandOptions());
		}
		unlink($tmpfile);
	}

	public function getCompleteConfig() {
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'configure',
				'show' ));
		if (FALSE === $this->exec()) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm('Configabruf/Backup der CRM ist gescheitert.', $this->getReturnCode(), $this->getCommand(), 'localhost', $this->getCommandOptions());
		}
		$config = implode("\n", $this->getOutput());
		//		$this->log()->debug(Hpls_Debug::print_r($config, "CRM BACKUP"));
		return $config;
	}

	public function restartResource($resID) {
		//crm resource restart resIMAP-Doveco
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'resource',
				'restart',
				$resID ));
		return $this->exec();
	}

	public function startResource($resID) {
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'resource',
				'start',
				$resID ));
		return $this->exec();
	
	}

	public function stopResource($resID) {
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'resource',
				'stop',
				$resID ));
		return $this->exec();
	
	}

	public function listResources() {
	
	}

	public function locateResource($resID) {
		$this->setCommand(self::CRM_RESOURCE_EXECUTABLE, array(
				'--locate',
				'--resource',
				"\"$resID\"" ));
		if (FALSE === $this->exec()) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm(__METHOD__, -1, $this->getCommand(), $this->getCommandOptions());
		}
		//		resource resTestIP is running on: mail-archiv1.heinlein-support.test
		$output = implode('', $this->getOutput());
		$o = explode('running on: ', $output);
		return trim($o[1]);
	}

	public function setResourceParameter($resId, $paramName, $value, $isMeta = false) {
		//		crm_resource --resource resIMAP-IP --set-parameter ip --parameter-value 172.16.5.136
		$options = array(
				'-r',
				$resId );
		$options[] = '-p';
		$options[] = $paramName;
		if ($isMeta) {
			$options[] = '--meta';
		}
		$options[] = '-v';
		$options[] = "\"$value\"";
		$this->setCommand(self::CRM_RESOURCE_EXECUTABLE, $options);
		return $this->exec();
	}

	public function setManagedResource($resID, $managed = true) {
		//		crm_resource -t primitive -r <rsc_id> -p is-managed -v false
		$this->setCommand(self::CRM_RESOURCE_EXECUTABLE, array(
				'-t',
				'primitive',
				'-r',
				$resID,
				'--meta',
				'-p',
				'is-managed',
				'-v',
				$managed ? 'true' : 'false' ));
		return $this->exec();
	}

	public function setManagedAll($managed = true) {
	
	}

	public function isResourceDefined($resID) {
	
	}

	/**
	 * @param unknown_type $resID
	 * @throws Hpls_Exception_Executable_Wrapper
	 */
	public function getResourceConfig($resID) {
		$this->setCommand(self::CRM_EXECUTABLE, array(
				'configure',
				'show',
				$resID ));
		if (FALSE === $this->exec()) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm(__METHOD__, -1, $this->getCommand(), $this->getCommandOptions());
		}
		return $this->getOutput();
	}

	public function getCrmMonHtml() {
		$config = Zend_Registry::get(MAConstsBack::REGBCK_CONFIG);
		
		$tmpFile = $config->cacheDir;
		$tmpFile .= '/crm_mon.html';
		$this->setCommand(self::CRM_MON_EXECUTABLE, array(
				'-1',
				'-nrft',
				'--as-html',
				$tmpFile ));
		if (FALSE === $this->exec()) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm("crm_mon > html output aufruf ist gescheitert.", $this->getReturnCode(), $this->getCommand(), $this->getCommandOptions());
		}
		$html = file_get_contents($tmpFile);
		if (FALSE === $html) {
			require_once 'Hpls/Exception/Executable/Wrapper/Crm.php';
			throw new Hpls_Exception_Executable_Wrapper_Crm("Konnte den HTML-Output von crm_mon nicht aus Datei '$tmpFile' lesen.", -2, $this->getCommand(), $this->getCommandOptions());
		}
		@unlink($tmpFile); //wird scheitern, da nötige dateirechte fehlen, aber egal
		return $html;
	}

	public function setExecWithSudo($value = true) {
		parent::setExecWithSudo($value);
	}

	public function isExecWithSudo() {
		return parent::isExecWithSudo();
	}

	/**
	 * @todo comment
	 * 
	 * @param string|NULL $host Der Hostname/IP, auf dem der Service manipuliert wurde, NULL => LOCALHOST
	 * @return multitype:multitype:string|null Die Ausgabe des Ausrufs auf Host $host.
	 */
	public function getOutput($host = null) {
		return parent::getOutput($host);
	}

	/**
	 * @todo comment
	 * 
	 * @param string|NULL $host Der Hostname/IP, null => LOCALHOST
	 * @return int|null Der Returncode des Ausrufs auf Host $host.
	 */
	public function getReturnCode($host = null) {
		return parent::getReturnCode($host);
	}

}