<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 19.12.2010
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 *
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Job/StatusAbstract.php';
require_once 'Zend/Config.php';
require_once 'Zend/Config/Writer/Ini.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage Job-Queue
 * @version $LastChangedRevision$
 * @since 1.0.7
 */
class Hpls_Job_Status_File extends Hpls_Job_StatusAbstract {
	const PERSIST_PATTERN = "%s--%s---.job.status";
	
	protected $_persistFileName = null;

	public function __construct($jobName, $jobID, $jobHost = 'localhost', $persistFileName = null) {
		parent::__construct($jobName, $jobID, $jobHost);
		
		if (is_null($persistFileName)) {
			$dir = Zend_Registry::get(Hpls_Consts0::REGK_PERSISTENCE)->hpls_jobs->status->file;
			$filename = sprintf(self::PERSIST_PATTERN, $this->getJobName(), $this->getJobID());
			$this->_persistFileName = $dir . DIRECTORY_SEPARATOR . $filename;
			Hpls_Debug::log()->debug(__METHOD__ . "## Persisting to file:" . $this->_persistFileName);
		}
	}

	/* (non-PHPdoc)
	 * @see Hpls_Job_StatusAbstract::save()
	 */
	public function save() {
		// TODO Auto-generated method stub
		$iniSteps = new Zend_Config(array(
				'job' => array(
						'name' => $this->_jobName,
						'id' => $this->_jobID,
						'host' => $this->_jobHost,
						'finished' => $this->_jobFinished ) ), true);
		$steps = array();
		foreach ($this->_steps as $k => $v) {
			$steps[$k] = $v->toArray();
		}
		$iniSteps->steps = $steps;
		$writer = new Zend_Config_Writer_Ini();
		$writer->write($this->_persistFileName, $iniSteps);
	
	}

	/* (non-PHPdoc)
	 * @see Hpls_Job_StatusAbstract::load()
	 */
	public function load() {
		// TODO Auto-generated method stub
		if (file_exists($this->_persistFileName)) {
			$ini = new Zend_Config_Ini($this->_persistFileName);
			$loadedSteps = array();
			$steps = $ini->steps->toArray();
			foreach ($steps as $k => $v) {
				$step = new Hpls_Job_Step($this, $v, $v['order']);
				$loadedSteps[$k] = $step;
			}
			
			if (count($this->_steps) > 0) {
				foreach ($loadedSteps as $key => $step) {
					if (array_key_exists($key, $this->_steps)) {
						$this->_steps[$key] = $step;
					} else {
						require_once 'Hpls/Exception/Job/Status.php';
						throw new Hpls_Exception_Job_Status("Step '$key' is not defined, but in persistence store.");
					}
				}
			}
			
			$this->_jobFinished = Hpls_Utils::toBool($ini->job->finished);
			$this->_loaded = true;
		
		//			if ($this->)
		//			$this->_currentStep =  $this->
		}
	}

	public function hasPersistedState() {
		return file_exists($this->_persistFileName);
	}

	/* (non-PHPdoc)
	 * @see Hpls_Job_StatusAbstract::clearPersistedState()
	 */
	public function clearPersistedState() {
		// TODO Auto-generated method stub
		if (file_exists($this->_persistFileName)) {
			return unlink($this->_persistFileName);
		}
	}

}