<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Apr 7, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2010-04-08 17:02:46 +0200 (Do, 08. Apr 2010) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 5135 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/HMT/GUI/Core/trunk/hpls.hmt.gui.libs/Hpls/Mailtrace/Utils.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
class Hpls_Mailtrace_Utils
{
	const REGEX_IPNAME = '/([^[]*)\[([^]]*)\](:(\d+))?/i';

	/**
	 * @todo comment
	 * Splits strings of form '<name>[<ip>]' into NAME and IP part.
	 * Returns an array with
	 * <ul>
	 * <li>['ip']: IP part of $serverstring
	 * <li>['name']: NAME part of $serverstring
	 * </ul>
	 * 
	 * @param string $serverstring
	 * @return NULL|NULL|multitype:string An array with
	 * <ul>
	 * <li>['ip']: IP part of $serverstring
	 * <li>['name']: NAME part of $serverstring
	 * </ul>
	 * or NULL if $serverstring is not a string or not of form '&lt;name&gt;[&lt;ip&gt;]'
	 */
	public static function splitIpName($serverstring) {
		if (is_null($serverstring) || !is_string($serverstring))
			return null;
		$res = preg_match('/([^[]*)\[([^]]*)\]/i', $serverstring, $matches);
		if ($res === false) {
			//error regex execution
			require_once 'Hpls/Mailtrace/Exception.php';
			throw new Hpls_Mailtrace_Exception(
			"Execution of regex '" . self::REGEX_IPNAME . "' on '$serverstring' failed.");
		}
		if ($res <= 0) {
			//no matches
			return null;
		}
		$ret = array('name' => $matches[1], 'ip' => $matches[2]);
		if (count($matches) >= 5) {
			$ret['port'] = $matches[4];
		}
		
//		require_once 'Hpls/Mailtrace.php';
//		require_once 'Zend/Debug.php';
//		Hpls_Mailtrace::i()->getLogger()->debug(Zend_Debug::dump($ret,"splitIpName '$serverstring':",false));
		
		return $ret;
	}
}