<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 29, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/Model/Abstract.php $
 *
 */
/***/
/**
 * @todo comment
 * A optional checked model, where all attributes stored in an array.
 * 
 * It is based on the magic functions from PHP5 and optionally checks if a attribute exists in the storage array.
 * If it does the value is set or returned otherwise a exception is thrown, if restriction is activated.
 * 
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage Model
 * @version $LastChangedRevision: 9196 $
 * @since alpha
 *
 */
abstract class Hpls_Model_Abstract {
	protected $_attributes = array();
	protected $_restrictAttrs = true;
	protected $_logger = NULL;

	protected function __construct($allowedAttrs = NULL, $isInitArray = false) {
		if (is_array($allowedAttrs)) {
			//initialize array
			foreach ($allowedAttrs as $attr) {
				$this->_attributes[$attr] = NULL;
			}
		} else {
			$this->_restrictAttrs = false;
		}
	}

	public function isAttrRestricted() {
		return $this->_restrictAttrs;
	}

	public function __get($varName) {
		if ($this->isAttrRestricted()) {
			if (array_key_exists($varName, $this->_attributes)) {
				return $this->_attributes[$varName];
			} else {
				require_once 'Hpls/Exception/ModelMissuse.php';
				throw new Hpls_Exception_ModelMissuse("Var '" . $varName . "' not avaiable in this model.");
			}
		} else {
			if (isset($this->_attributes[$varName])) {
				return $this->_attributes;
			} else {
				return NULL;
			}
		}
	}

	public function __set($varName, $value) {
		if ($this->isAllowedAttr($varName)) {
			$this->_attributes[$varName] = $value;
		}
	}

	public function __isset($varName) {
		return isset($this->_attributes[$varName]);
	}

	/**
	 * @todo comment
	 * @todo think over: currently it prevents setting the variable again, if restrictions is on
	 * @param unknown_type $varName
	 */
	public function __unset($varName) {
		if (isset($this->_attributes[$varname])) {
			unset($this->_attributes[$varname]);
		}
	}

	final protected function get($varName) {
		return $this->__get($varName);
	}

	final protected function set($varName, $value) {
		if ($this->isAllowedAttr($varName)) {
			$this->_attributes[$varName] = $value;
		}
	}

	public function setValues(array $values) {
		if (is_null($values))
			return;
		
		foreach ($values as $key => $value) {
			$this->__set($key, $value);
		}
	}

	protected function isAllowedAttr($varName) {
		return (!$this->isAttrRestricted()) || array_key_exists($varName, $this->_attributes);
	}

	protected function getLogger() {
		if (is_null($this->_logger)) {
		
		}
		return $this->_logger;
	}
}