<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 28.04.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Utils/Logusing.php';
/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
abstract class Hpls_SettingsAbstract extends Hpls_Utils_Logusing {
	
	/**
	 * @todo comment
	 * @var Hpls_Settings|Hpls_SettingsAbstract
	 */
	protected $_parent = null;
	protected $_saveSelf = false;
	protected $_children = array();
	protected $_key = null;
	protected $_values = array();
	protected $_changedValues = array();
	protected $_allowedValues = null;

	public function __construct($parent, $allowedProperties = null, $key = null) {
		if (!($parent instanceof Hpls_SettingsAbstract || $parent instanceof Hpls_Settings)) {
			throw new InvalidArgumentException('$parent must be of type Hpls_SettingsAbstract or Hpls_Settings', 1);
		}
		$this->_parent = $parent;
		$this->_allowedValues = $allowedProperties;
		$this->_key = $key;
	}

	public function __call($name, $arguments) {
		if (stripos($name, 'get') === 0) {
			$valueKey = strtolower(substr($name, 3));
			if (isset($this->_values[$valueKey]))
				return $this->_values[$valueKey];
			else {
				return null;
			}
		}
		if (stripos($name, 'set') === 0) {
		
		}
	}

	public function __get($name) {
		$valueKey = strtolower($name);
		if (isset($this->_values[$valueKey]))
			return $this->_values[$valueKey];
		return null;
	}

	public function __set($name, $value) {
		$valueKey = strtolower($name);
		$this->_values[$valueKey] = $value;
	}

	public function getKey() {
		return $this->_key;
	}

	public function setKey($key) {
		$this->_key = $key;
	}

	public function setValues(array $values) {
		foreach ($values as $k => $v) {
			$this->_values[$k] = $v;
		}
	}

	public function isSaveSelf() {
		return $this->_saveSelf;
	}

	public function setSaveSelf($v = true) {
		$this->_saveSelf = $v;
	}

	public function save() {
		$this->_parent->save();
	}

	public function load() {
		$this->_parent->load();
	}

	public function addChild(Hpls_SettingsAbstract $child) {
		$this->_children[$child->getKey()] = $child;
	}

	public function getChild($key) {
		return isset($this->_children[$key]) ? $this->_children[$key] : null;
	}

	public function getChildren() {
		return $this->_children;
	}

	public function toArray() {
		$ret = $this->_values;
		foreach ($this->_children as $key => $child) {
			$ret[$key] = $child->toArray();
		}
		return $ret;
	}

	protected function setChanged($key) {
		$this->_changedValues[$key] = time();
	}

}