<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 21.06.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Var/Matcher.php';
/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Storage_Reader_Ldap_Attributereader {
	protected $_query = null;
	protected $_fields = null;
	protected $_name = null;
	protected $_ldapCon = null;
	
	/**
	 * @todo comment
	 * @var array:Hpls_Var_Matcher
	 */
	protected $_valuefilters = array();

	/**
	 * @todo comment
	 * @param unknown_type $name
	 * @param array $options
	 * @return Hpls_Storage_Reader_Ldap_Attributereader
	 */
	public function __construct($name, array $options) {
		
		$this->_name = $name;
		if (isset($options['fields'])) {
			if (!is_array($options['fields'])) {
				$options['fields'] = explode(',', $options['fields']);
			}
			$this->_fields = array();
			foreach ($options['fields'] as $field) {
				$f = strtolower(trim($field));
				$this->_fields[$f] = $f;
			}
		}
		if (isset($options['query']) && is_array($options['query'])) {
			$this->_query['filter'] = $options['query']['filter'];
			$this->_query['scope'] = isset($options['query']['scope']) ? $options['query']['scope'] : 'sub';
		}
		if (isset($options['valuefilter']) && is_array($options['valuefilter'])) {
			//ein oder mehrere valuefilter? <- unterscheidbar durch ebene, an der 'match' zu finden ist.
			if (isset($options['valuefilter']['match'])) {
				$this->addFilter(new Hpls_Var_Matcher($options['valuefilter']));
			} else {
				foreach ($options['valuefilter'] as $vfilter) {
					$this->addFilter(new Hpls_Var_Matcher($vfilter));
				}
			}
		}
	}

	public function addFilter(Hpls_Var_Matcher $filter) {
		$this->_valuefilters[] = $filter;
	}

	public function read($ldapCon, $makeDistinct = true) {
		$this->setLdapCon($ldapCon);
		$ret = array();
		$baseDn = $this->getBaseDn();
		$filter = $this->getQuery();
		$scope = $this->getScope();
		$fieldNames = array_values($this->getFieldNames());
		isDebug('user_email_read') && $this->log()->debug(__METHOD__ . " ldap_search('$scope' ,'$baseDn', '$filter', " . print_r($fieldNames, true) . ')');
		$resID = false;
		switch ($scope) {
			case 'base' :
				$resID = ldap_list($this->getLdapCon(), $baseDn, $filter, $fieldNames);
				break;
			case 'list' :
			case 'one' :
				$resID = ldap_list($this->getLdapCon(), $baseDn, $filter, $fieldNames);
				break;
			case 'sub' :
			default :
				$resID = ldap_search($this->getLdapCon(), $baseDn, $filter, $fieldNames);
				break;
		}
		if (false === $resID) {
			return false;
		}
		$entries = ldap_get_entries($this->_ldapCon, $resID);
		if (false === $entries) {
			return false;
		}
		isDebug('ldap_read', 'user_email_read') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($entries, ' READ RAW ENTRIES'));
		foreach ($entries as $entry) {
			foreach ($this->_valuefilters as $vf) {
				foreach ($fieldNames as $field) {
					if (isset($entry[$field]) && $entry[$field]['count'] > 0) {
						foreach ($entry[$field] as $k => $fvalue) {
							if ($k === 'count')
								continue;
							
							if ($makeDistinct) {
								$fvalue = $vf->getSubstituted(null, $fvalue);
								$ret[$fvalue] = $fvalue;
							} else {
								$ret[] = $vf->getSubtituted(null, $fvalue);
							} //if
						}
					} //if
				}
			}
		}
		isDebug('ldap_read', 'user_email_read') && $this->log()->debug(__METHOD__ . Hpls_Debug::print_r($ret, 'READ TRANSFORMED ENTRIES'));
		return $ret;
	}

	public function setQuery($filter) {
		$this->_query['filter'] = $filter;
	}

	public function setScope($scope) {
		$this->_query['scope'] = $scope;
	}

	public function setBaseDn($baseDn) {
		$this->_query['basedn'] = $baseDn;
	}

	/**
	 * @return the $_ldapCon
	 */
	public function getLdapCon() {
		return $this->_ldapCon;
	}

	/**
	 * @param field_type $_ldapCon
	 */
	public function setLdapCon($ldapCon) {
		$this->_ldapCon = $ldapCon;
	}

	public function getQuery() {
		return empty($this->_query['filter']) ? '' : $this->_query['filter'];
	}

	public function getScope() {
		return $this->_query['scope'];
	}

	public function getBaseDn() {
		return $this->_query['basedn'];
	}

	public function getFieldNames() {
		return $this->_fields;
	}

	/**
	 * @return the $_name
	 */
	public function getName() {
		return $this->_name;
	}

	/**
	 * @param field_type $_name
	 */
	public function setName($_name) {
		$this->_name = $_name;
	}

	protected function log() {
		return Hpls_Utils::log();
	}

}