<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created        : 21.06.2011
 * created by     : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate$
 * last changed by: $LastChangedBy$
 * revision       : $LastChangedRevision$
 * SVN head URL   : $HeadURL$
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Hpls/Storage/Reader/Ldap/Attributereader.php';
/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package ??
 * @subpackage ??
 * @version $LastChangedRevision$
 * @since 1.0.9
 */
class Hpls_Storage_Reader_Ldap_Basedsearch {
	protected $_type = null;
	protected $_match = array();
	protected $_filter = null;
	protected $_attribute = array();
	protected $_ldapCon = null;
	protected $_continue = true;
	/**
	 * BaseDN, ab der die eigentlichen Attribute gelesen werden sollen. Wird im ersten Schritt bestimmt.
	 * @var string
	 */
	protected $_startBaseDn = null;
	/**
	 * @todo comment
	 * @var Hpls_Var_Subster
	 */
	protected $_variables = null;

	/**
	 * @todo comment
	 * @param array $options
	 * @return Hpls_Storage_Reader_Ldap_Basedsearch
	 */
	public function __construct(array $options, $variables = array()) {
		isDebug('method_trace') && $this->log()->debug(Hpls_Debug::print_r($options, __METHOD__ . " OPTIONS"));
		$this->_variables = new Hpls_Var_Subster($variables);
		if (isset($options['type'])) {
			$this->_type = (string) $options['type'];
		}
		if (isset($options['continue'])) {
			$this->_continue = Hpls_Utils::toBool($options['continue']);
		}
		if (isset($options['match'])) {
			$match = $options['match'];
			
			if (isset($match['query']) && is_array($match['query'])) {
				//@todo error handling : kein filter!
				$this->_match['filter'] = isset($match['query']['filter']) ? $match['query']['filter'] : null;
				$this->_match['scope'] = isset($match['query']['scope']) ? $match['query']['scope'] : 'sub';
				$this->_match['basedn'] = isset($match['query']['basedn']) ? $match['query']['basedn'] : null;
			}
		}
		if (isset($options['read'])) {
			$readOpt = $options['read'];
			if (isset($readOpt['attribute']) && is_array($readOpt['attribute'])) {
				foreach ($readOpt['attribute'] as $aname => $readOpts) {
					if (is_array($readOpts)) {
						Hpls_Utils::log()->debug(Hpls_Debug::print_r($readOpts, __METHOD__));
						$attrRead = new Hpls_Storage_Reader_Ldap_Attributereader($aname, $readOpts);
						$this->addAttributeReader($attrRead);
					}
				}
			}
		}
		return $this;
	}

	/**
	 * @todo comment
	 * @return boolean
	 */
	public function match() {
		
		$baseDn = $this->getMatchBaseDN();
		$filter = $this->getMatchFilter();
		$scope = $this->getMatchScope();
		isDebug('method_trace') && $this->log()->debug(__METHOD__ . ": searching in basedn: $baseDn for $filter with scope $scope");
		$resID = false;
		switch ($scope) {
			case 'base' :
				$resID = ldap_read($this->getLdapCon(), $baseDn, $filter);
				break;
			case 'list' :
			case 'one' :
				$resID = ldap_list($this->getLdapCon(), $baseDn, $filter);
				break;
			case 'sub' :
			default :
				$resID = ldap_search($this->getLdapCon(), $baseDn, $filter);
				break;
		}
		if (false === $resID) {
			isDebug('method_trace') && $this->log()->debug(__METHOD__ . ": no match");
			return false;
		}
		//der erste gefundene eintrag ist die neue basedn
		if (ldap_count_entries($this->getLdapCon(), $resID) > 0) {
			$this->setStartBaseDn(ldap_get_dn($this->getLdapCon(), ldap_first_entry($this->getLdapCon(), $resID)));
			isDebug('method_trace') && $this->log()->debug(__METHOD__ . ": new start baseDn: " . $this->getStartBaseDn());
			return true;
		}
		return false;
	}

	public function getAttributes() {
		$ret = array();
		foreach ($this->_attribute as $name => /**
		 * @var Hpls_Storage_Reader_Ldap_Attributereader
		 */
		$attrRead) {
			$attrRead->setBaseDn($this->getStartBaseDn());
			
			$ret[$name] = $attrRead->read($this->getLdapCon(), true);
		}
		return $ret;
	}

	public function getAttribute($virtualName) {
		isDebug() && $this->log()->debug(Hpls_Debug::print_r($virtualName, __METHOD__));
		if (isset($this->_attribute[$virtualName])) {
			$attrRead = $this->_attribute[$virtualName];
			$attrRead->setBaseDn($this->getStartBaseDn());
			isDebug() && $this->log()->debug(Hpls_Debug::print_r($attrRead, __METHOD__ . ' ATTR READER'));
			return $attrRead->read($this->getLdapCon(), true);
		}
	}

	public function addAttributeReader(Hpls_Storage_Reader_Ldap_Attributereader $reader) {
		$this->_attribute[$reader->getName()] = $reader;
	}

	/**
	 * @return the $_startBaseDn
	 */
	public function getStartBaseDn() {
		return $this->_startBaseDn;
	}

	/**
	 * @return the $_ldapCon
	 */
	public function getLdapCon() {
		return $this->_ldapCon;
	}

	/**
	 * @param field_type $_ldapCon
	 */
	public function setLdapCon($ldapCon) {
		$this->_ldapCon = $ldapCon;
	}

	public function setVariable($name, $value) {
		$this->_variables->setVariable($name, $value);
	}

	protected function getMatchBaseDN() {
		return $this->_variables->getSubstituted($this->_match['basedn']);
	}

	protected function getMatchFilter() {
		return $this->_variables->getSubstituted($this->_match['filter']);
	}

	protected function getMatchScope() {
		return $this->_match['scope'];
	}

	/**
	 * @param string $_startBaseDn
	 */
	protected function setStartBaseDn($_startBaseDn) {
		$this->_startBaseDn = $_startBaseDn;
	}

	/**
	 * @return the $_continue
	 */
	public function isContinue() {
		return $this->_continue;
	}

	/**
	 * @param field_type $_continue
	 */
	public function setContinue($_continue) {
		$this->_continue = $_continue;
	}

	protected function log() {
		return Hpls_Utils::log();
	}

}