<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Feb 26, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs/Hpls/View/Paginator/Paginator.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 *
 */
/***/
require_once 'Hpls/Model/Abstract.php';
require_once 'Hpls/View/Paginator/Interface.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HPLS
 * @subpackage View
 * @version $LastChangedRevision: 9196 $
 * @since beta
 *
 */
class Hpls_View_Paginator_Paginator extends Hpls_Model_Abstract implements 
	Hpls_View_Paginator_Interface {
	protected static $ALLOWED_VALUES = array(
			self::ITEM_LIMIT,
			self::PAGING_RANGE,
			self::PAGING_FIRST,
			self::PAGING_LAST,
			self::PAGE_FIRST,
			self::PAGE_LAST,
			self::PAGE_CURRENT,
			self::PAGE_NEXT,
			self::PAGE_PREV,
			self::PAGE_LINK_BASE,
			self::ITEM_FIRST,
			self::ITEM_LAST,
			self::ITEM_COUNT );
	protected static $NEEDED_VALUES = array(
			self::ITEM_COUNT,
			self::ITEM_LIMIT,
			self::PAGE_CURRENT,
			self::PAGING_RANGE );

	function __construct(array $values = NULL) {
		parent::__construct(self::$ALLOWED_VALUES);
		if (is_null($values)) {
			$this->set(self::ITEM_LIMIT, 20);
			$this->set(self::PAGING_RANGE, 2);
			$this->set(self::PAGE_FIRST, 1);
			$this->set(self::ITEM_FIRST, 1);
		} else {
			parent::setValues($values);
		}
	
		//DEBUG
	}

	public function __set($varName, $value) {
		if (array_key_exists($varName, self::$NEEDED_VALUES)) {
			if (method_exists($this, "set" . ucfirst($varName))) {
				$setMethod = "set" . ucfirst($varName);
				return $this->$setMethod($value);
			} else {
				return parent::__set($varName, $value);
			}
		} else {
			require_once 'Hpls/Exception/Paginator.php';
			throw new Hpls_Exception_Paginator("'$varName' is not settable or not a paginator property");
		}
	}

	public function __get($varName) {
		if ('Link' == substr($varName, -4)) {
			$varName = substr($varName, 0, strlen($varName) - 4);
			return $this->getLink($varName);
		}
		if (method_exists($this, "get" . ucfirst($varName))) {
			$getMethod = "get" . ucfirst($varName);
			return $this->$getMethod();
		} else {
			return parent::__get($varName);
		}
	}

	public function needsScrolling() {
		return !empty($this->pageLast) && !empty($this->pagingRange) && ($this->pageLast > $this->pagingRange);
	}

	public function setPageCurrent($pageNum) {
		if (empty($pageNum)) { //clear
		}
		if ($pageNum <= 1)
			$pageNum = 1;
		if (!empty($this->itemLimit) && !empty($this->itemCount)) {
			if (($this->itemLimit * $pageNum) > $this->itemCount) {
				$pageNum = ceil($this->itemCount / $this->itemLimit);
			}
		}
		$this->set(self::PAGE_CURRENT, $pageNum);
		$this->calcValues();
	}

	public function setItemLimit($value) {
		if ($value === 0) {
			throw new Hpls_Exception_Paginator("An itemLimit of 0 is not supported.");
		}
		$this->set(self::ITEM_LIMIT, $value);
		$this->calcValues();
	}

	public function setPagingRange($value) {
		$this->set(self::PAGING_RANGE, $value);
		$this->calcValues();
	}

	public function setItemCount($value) {
		$this->set(self::ITEM_COUNT, $value);
		$this->calcValues();
	}

	/**
	 * Is there a page left out between firstPage and firstPageInRange?
	 */
	public function isFirstContinous() {
		return !empty($this->pageFirst) && !empty($this->pagingFirst) && !($this->pageFirst + 1 < $this->pagingFirst);
		;
	}

	public function isLastContinous() {
		return !empty($this->pageLast) && !empty($this->pagingLast) && !($this->pageLast - 1 > $this->pagingLast);
		;
	}

	public function isCurrentPage($pageNum) {
		if ($this->pageCurrent == $pageNum) {
			return true;
		}
		return false;
	}

	/**
	 * @param unknown_type $pageNum
	 */
	public function getLink($pageNum) {
		if (empty($pageNum)) {
			require_once 'Hpls/Exception/Paginator.php';
			throw new Hpls_Exception_Paginator("No page given to generate link for.");
		}
		if (is_int($pageNum)) {
			return $this->genLink($pageNum);
		} else {
			return $this->genLink($this->{$pageNum});
		}
	}

	protected function genLink($pageNum) {
		if (empty($pageNum)) {
			require_once 'Hpls/Exception/Paginator.php';
			throw new Hpls_Exception_Paginator("No page given to generate link for.");
		}
		$baseLink = $this->{self::PAGE_LINK_BASE};
		if (!empty($baseLink)) {
			return preg_replace(array(
					'/\[\[PAGE\]\]/' ), array(
					$pageNum ), $baseLink);
		} else {
			return $pageNum;
		}
	}

	public function setPageLinkBase($url) {
		$this->set(self::PAGE_LINK_BASE, $url);
	}

	/**
	 * @return array 
	 */
	public function getPagesInRange() {
		$ret = array();
		if (!empty($this->pagingFirst) && !empty($this->pagingLast)) {
			for ($i = $this->pagingFirst; $i <= $this->pagingLast; $i++) {
				$ret[] = $i;
			}
		}
		return $ret;
	}

	/**
	 * @todo error handling
	 */
	protected function calcValues() {
		$pageFirst = $this->pageFirst;
		$pageLast = $this->pageLast;
		$itemCount = $this->itemCount;
		$pagingRange = $this->pagingRange;
		$itemLimit = $this->itemLimit;
		//pageLast
		if (!empty($this->itemLimit)) {
			$itemLimit = $this->itemLimit;
			if (!empty($this->itemCount)) {
				$itemCount = $this->itemCount;
				$pageLast = ($itemCount == 0 || $itemLimit == 0) ? 0 : ceil($itemCount / $itemLimit);
				$this->set(self::PAGE_LAST, $pageLast);
			}
		}
		//prev + next + itemfirst + itemlast
		if (!empty($this->pageCurrent)) {
			$pageCurrent = $this->pageCurrent;
			if ($pageCurrent > $pageFirst) {
				$this->set(self::PAGE_PREV, $pageCurrent - 1);
			} else {
				$this->set(self::PAGE_PREV, null);
			}
			if (!empty($pageLast) && $pageCurrent < $pageLast) {
				$this->set(self::PAGE_NEXT, $pageCurrent + 1);
			} else {
				$this->set(self::PAGE_NEXT, null);
			}
			//itemFirst, itemLast
			$this->set(self::ITEM_FIRST, ($itemLimit * ($pageCurrent - 1)) + 1);
			$this->set(self::ITEM_LAST, min($itemLimit * ($pageCurrent), $itemCount));
			//paging start
			//pagin end
			$pagingStart = max($pageCurrent - $pagingRange, 1);
			$pagingEnd = min($pageCurrent + $pagingRange, $pageLast);
			
			// if more pages then direct links to show
			$leftExtent = $pageCurrent - $pagingRange - $pageFirst;
			$rightExtent = $pageCurrent + $pagingRange - $pageLast;
			if ($leftExtent < 0) {
				$pagingEnd = min($pagingEnd + abs($leftExtent), $pageLast);
			}
			if ($rightExtent > 0) {
				$pagingStart = max($pagingStart - $rightExtent, $pageFirst);
			}
			
			$this->set(self::PAGING_FIRST, $pagingStart);
			$this->set(self::PAGING_LAST, $pagingEnd);
		} else {
			$this->set(self::PAGE_PREV, null);
			$this->set(self::PAGE_NEXT, null);
			$this->set(self::PAGING_FIRST, null);
			$this->set(self::PAGING_LAST, null);
			$this->set(self::ITEM_FIRST, null);
			$this->set(self::ITEM_LAST, null);
		}
		/*if (!$this->checkNeeded()) {
			return;
		}*/
	}

	protected function checkNeeded() {
		foreach (self::$NEEDED_VALUES as $need) {
			if (empty($this->{$need})) {
				return false;
			}
		}
		return true;
	}
}