<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * created   : Jan 19, 2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.libs.smarty/Hpls/View/Smarty.php $
 *
 */
/**
 *
 */
require_once 'Zend/View/Abstract.php';
require_once 'Zend/Registry.php';
require_once 'Hpls/Smarty/Smarty.php';
require_once 'Hpls/View/Consts.php';

/**
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package HMT
 * @subpackage ??
 * @version $LastChangedRevision: 9196 $
 * @since alpha
 */
class Hpls_View_Smarty extends Zend_View_Abstract {
	/**
	 * Smarty instance
	 *
	 * @var Hpls_Smarty_Smarty
	 */
	protected $_smarty = null;
	/**
	 * Template explicitly set to render in this view
	 * @var string
	 */
	protected $_customTemplate = '';
	
	/**
	 * Smarty config
	 * @var array
	 */
	private $_config = null;

	/**
	 * Class definition and constructor
	 *
	 * Let's start with the class definition and the constructor part.
	 * My class Travello_View_Smarty is extending the Zend_View_Abstract class.
	 * In the constructor the parent constructor from Zend_View_Abstract is called first.
	 * After that a Smarty object is instantiated, configured and stored in a private attribute.
	 * Please note that I use a configuration object from the object store to get the configuration data for Smarty.
	 *
	 * @param array $smartyConfig
	 * @param array $config
	 */
	public function __construct($config = array()) {
		if (!class_exists('Hpls_Smarty_Smarty', true)) {
			require_once 'Hpls/Smarty/Smarty.php';
		}
		
		$this->_smarty = new Hpls_Smarty_Smarty();
		
		$this->_config = $config;
		parent::__construct($config);
		$this->_configureSmarty();
	}

	/**
	 * Magic clone method, on clone create diferent smarty object
	 */
	public function __clone() {
		$this->_smarty = new Hpls_Smarty_Smarty();
		
		$this->_configureSmarty();
	}

	/**
	 * Return the template engine object
	 *
	 * @return Smarty
	 */
	public function getEngine() {
		return $this->_smarty;
	}

	/**
	 * Ustawia ścieżkę do szablonów
	 *
	 * @param string $path ścieżka
	 */
	public function setScriptPath($path) {
		parent::setScriptPath($path);
		$this->setTemplateDir($this->getScriptPaths());
	}

	public function addScriptPath($path) {
		parent::addScriptPath($path);
		$this->setTemplateDir($this->getScriptPaths());
	}

	/**
	 * Updates Smarty's template_dir field with new value
	 *
	 * @param string $dir
	 * @return Ext_View_Smarty
	 */
	public function setTemplateDir($dir) {
		$this->_smarty->setTemplateDir($dir);
		return $this;
	}

	/**
	 * Adds another Smarty template_dir to scan for templates
	 *
	 * @param string $dir
	 * @return Ext_View_Smarty
	 */
	public function addTemplateDir($dir) {
		$this->_smarty->addTemplateDir($dir);
		return $this;
	}

	/**
	 * Ustawienie parametru
	 *
	 * @param string $key klucz
	 * @param mixed $value wartość
	 */
	public function setParam($key, $value) {
		$this->_smarty->{$key} = $value;
	}

	/**
	 * Template getter (return file path)
	 * @return string
	 */
	public function getTemplate() {
		return $this->_customTemplate;
	}

	/**
	 * Template filename setter
	 * @param string
	 * @return Ext_View_Smarty
	 */
	public function setTemplate($tpl) {
		$this->_customTemplate = $tpl;
		return $this;
	}

	/**
	 * Magic setter for Zend_View compatibility. Performs assign()
	 *
	 * @param string $key
	 * @param mixed $val
	 */
	public function __set($key, $val) {
		$this->assign($key, $val);
	}

	/**
	 * Magic getter for Zend_View compatibility. Retrieves template var
	 *
	 * @param string $key
	 * @return mixed
	 */
	public function __get($key) {
		return $this->_smarty->getTemplateVars($key);
	}

	/**
	 * Magic getter for Zend_View compatibility. Removes template var
	 *
	 * @see View/Zend_View_Abstract::__unset()
	 * @param string $key
	 */
	public function __unset($key) {
		$this->_smarty->clearAssign($key);
	}

	/**
	 * Allows testing with empty() and isset() to work
	 * Zend_View compatibility. Checks template var for existance
	 *
	 * @param string $key
	 * @return boolean
	 */
	public function __isset($key) {
		return (null !== $this->_smarty->getTemplateVars($key));
	}

	public function set($key, $value) {
		return $this->assign($key, $value);
	}

	public function get($key) {
		return $this->__get($key);
	}

	/**
	 * Overwrite assign() method
	 *
	 * The next part is an overwrite of the assign() method from Zend_View_Abstract, which works in a similar way. The big difference is that the values are assigned to the Smarty object and not to the $this->_vars variables array of Zend_View_Abstract.
	 *
	 * @param string|array $var
	 * @return Ext_View_Smarty
	 */
	public function assign($var, $value = null, $scope = Smarty::SCOPE_GLOBAL) {
		if (is_string($var)) {
			$this->_smarty->assign($var, $value, false, $scope);
		} elseif (is_array($var)) {
			foreach ($var as $key => $value) {
				$this->assign($key, $value, false, $scope);
			}
		} else {
			throw new Zend_View_Exception('assign() expects a string or array, got ' . gettype($var));
		}
		return $this;
	}

	/**
	 * @todo comment
	 * Alias for append()
	 *
	 * @param unknown_type $var
	 * @param unknown_type $value
	 * @param unknown_type $merge
	 * @param unknown_type $nocache
	 * @param unknown_type $scope
	 * @return unknown_type
	 */
	public function addValue($var, $value = null, $merge = false, $nocache = false, $scope = Smarty::SCOPE_GLOBAL) {
		$this->append($var, $value, $merge, $nocache, $scope);
	}

	/**
	 * appends values to template variables
	 *
	 * @param array|string $var the template variable name(s)
	 * @param mixed $value the value to append
	 * @param boolean $merge flag if array elements shall be merged
	 * @param boolean $nocache if true any output of this variable will be not cached
	 * @param boolean $scope the scope the variable will have  (local,parent or root)
	 */
	public function append($var, $value = null, $merge = false, $nocache = false, $scope = Smarty::SCOPE_GLOBAL) {
		$this->_smarty->append($var, $value, $merge, $nocache, $scope);
	}

	/**
	 * Overwrite escape() method
	 *
	 * The next part is an overwrite of the escape() method from Zend_View_Abstract. It works both for string and array values and also uses the escape() method from the Zend_View_Abstract. The advantage of this is that I don't have to care about each value of an array to get properly escaped.
	 *
	 * @param mixed $var
	 * @return mixed
	 */
	public function escape($var) {
		if (is_string($var)) {
			return parent::escape($var);
		} elseif (is_array($var)) {
			foreach ($var as $key => $val) {
				$var[$key] = $this->escape($val);
			}
		}
		return $var;
	}

	/**
	 * Alias for append()
	 *
	 * @param string $tpl_var the template variable name
	 * @param mixed $ &$value the referenced value to append
	 * @param boolean $merge flag if array elements shall be merged
	 */
	public function addValue_by_ref($tpl_var, &$value, $merge = false) {
		$this->append_by_ref($tpl_var, $value, $merge);
	}

	/**
	 * appends values to template variables by reference
	 *
	 * @param string $tpl_var the template variable name
	 * @param mixed $ &$value the referenced value to append
	 * @param boolean $merge flag if array elements shall be merged
	 */
	public function append_by_ref($tpl_var, &$value, $merge = false) {
		$this->_smarty->appendByRef($tpl_var, $value, $merge);
	}

	/**
	 * Zend_View compatibility. Retrieves all template vars
	 *
	 * @see Zend_View_Abstract::getVars()
	 * @return array
	 */
	public function getVars() {
		return $this->_smarty->getTemplateVars();
	}

	/**
	 * Zend_View compatibility. Removes all template vars
	 *
	 * @see View/Zend_View_Abstract::clearVars()
	 * @return Ext_View_Smarty
	 */
	public function clearVars() {
		$this->_smarty->clearAllAssign();
		$this->assign('this', $this);
		return $this;
	}

	//	public function render($name) {
	//		Zend_Registry::get(Hpls_Consts::REGK_LOGGER)->debug("Rendering template: " . $name);
	//		return $this->_smarty->fetch($name);
	//	}
	

	/**
	 * Print the output
	 *
	 * The next method output() is a wrapper on the render() method from Zend_View_Abstract. It just sets some headers before printing the output.
	 *
	 * @param <type> $name
	 */
	public function output($name) {
		header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
		header("Cache-Control: no-cache");
		header("Pragma: no-cache");
		header("Cache-Control: post-check=0, pre-check=0", false);
		
		print $this->render($name);
	}

	/**
	 * Use Smarty caching
	 *
	 * The last two methods were created to simply integrate the Smarty caching mechanism in the View class.
	 * With the first one you can check for cached template and with the second one you can set the caching on or of.
	 *
	 * @param string $template
	 * @return bool
	 */
	public function isCached($template) {
		return $this->_smarty->is_cached($template);
	}

	/**
	 * Enable/disable caching
	 *
	 * @param bool $caching
	 * @return Hpls_View_Smarty
	 */
	public function setCaching($caching) {
		$this->_smarty->caching = $caching;
		return $this;
	}

	/**
	 * Adds another Smarty plugin directory to scan for plugins
	 *
	 * @param string $dir
	 * @return Ext_View_Smarty
	 */
	public function addPluginDir($dir) {
		$this->_smarty->addPluginsDir($dir);
		return $this;
	}

	/**
	 * Zend_View compatibility. Add the templates dir
	 *
	 * @see View/Zend_View_Abstract::addBasePath()
	 * @return Ext_View_Smarty
	 */
	public function addBasePath($path, $classPrefix = 'Zend_View') {
		parent::addBasePath($path, $classPrefix);
		$this->addScriptPath($path . '/templates');
		//		$this->addTemplateDir($path . '/templates/static');
		return $this;
	}

	/**
	 * Zend_View compatibility. Set the templates dir instead of scripts
	 *
	 * @see View/Zend_View_Abstract::setBasePath()
	 * @return Ext_View_Smarty
	 */
	public function setBasePath($path, $classPrefix = 'Zend_View') {
		parent::setBasePath($path, $classPrefix);
		$this->setScriptPath($path . '/templates');
		//		$this->addTemplateDir($path . '/templates/static');
		return $this;
	}

	/**
	 * @todo comment
	 *
	 * Variables:
	 * - table[]
	 * - [headers][]
	 * - [title] => i18n-string-id
	 * - [sortable] => url to sort after
	 * - [active] => bool
	 * - [items][]
	 * - [row*]
	 * - ([active]) => bool
	 * - ([link]) => url or array (module,controller,action,params)
	 * - [values] => mixed
	 * - [value] =>
	 * - [doI18N] => bool
	 *}
	 *
	 * @param unknown_type $tableID
	 * @param array $header
	 * @param array $rows
	 * @return unknown_type
	 */
	public function appendTable($tableID, array $header, array $rows) {
		$ret = array(
				HVConsts::V_T_HEAD => $header,
				HVConsts::V_T_BODY => $rows );
		$this->append(HVConsts::V_TABLES, $ret);
	}

	public function appendToTable() {
	}

	public function appendMenu($menuID, array $menu) {
		$menu = array();
		$menu[Hpls_View_Consts::MENU_TOP] = $this->generateTopMenu();
		$menu[Hpls_View_Consts::MENU_LEFT] = $this->generateLeftMenu();
		$this->assign(Hpls_View_Consts::MENU, $menu);
	}

	public function addForm(Zend_Form $formDef) {
		if (empty($formID) || empty($formDef)) {
			return;
		}
		
		$this->append(HVConsts::V_FORMS, $formDef);
	}

	/**
	 * @todo comment
	 * @param unknown_type $componentID
	 * @param string|array $errorFieldID
	 * @param unknown_type $value
	 */
	public function addErrorText($componentID, $errorFieldID, $value = null) {
		if ($this->_smarty->isSetVariable(HVConsts::V_ERRORS)) {
			$var = $this->_smarty->getVariable(HVConsts::V_ERRORS);
			$var = $var->value;
		} else {
			$var = array();
		}
		if (is_array($errorFieldID)) {
			Hpls_Utils::getLogger()->debug("ERR-array: " . print_r($errorFieldID, true));
			$val = array();
			if (isset($var[$componentID])) {
				$val = $var[$componentID];
			}
			
			foreach ($errorFieldID as $k => $v) {
				$val = array_merge($val, array(
						$k => $v ));
				Hpls_Utils::getLogger()->debug("ERR-val-array: " . print_r($val, true));
			}
			$var[$componentID] = $val;
		} else {
			if (isset($var[$componentID])) {
				$val = $var[$componentID];
				$val = array_merge($val, array(
						$errorFieldID => $value ));
				$var[$componentID] = $val;
			} else {
				$var[$componentID] = array(
						$errorFieldID => $value );
			}
		}
		$this->assign(HVConsts::V_ERRORS, $var, true);
	}

	/**
	 * Initializes the smarty and populates config params
	 *
	 * @throws Zend_View_Exception
	 * @return void
	 */
	private function _configureSmarty() {
		
		if ($this->_config === null) {
			require_once 'Hpls/Exception/View.php';
			throw new Hpls_Exception_View("Could not locate Smarty config - node 'smarty' not found");
		}
		$config = $this->_config;
		
		$this->_smarty->setZendView($this);
		if (isset($config['params'])) {
			foreach ($config['params'] as $key => $value) {
				$this->_smarty->{$key} = $value;
			}
		}
		
		if (array_key_exists('caching', $config)) {
			$this->_smarty->caching = $config['caching'];
		}
		if (array_key_exists('cache_lifetime', $config)) {
			$this->_smarty->cache_lifetime = $config['cache_lifetime'];
		}
		if (array_key_exists('cache_dir', $config)) {
			$this->_smarty->cache_dir = $config['cache_dir'];
		}
		//		$this->_smarty->template_dir = $this->_config['template_dir'];
		if (array_key_exists('compile_dir', $config)) {
			$this->_smarty->compile_dir = $config['compile_dir'];
		}
		if (array_key_exists('config_dir', $config)) {
			$this->_smarty->config_dir = $config['config_dir'];
		}
		if (array_key_exists('left_delimiter', $config)) {
			$this->_smarty->left_delimiter = $config['left_delimiter'];
		}
		if (array_key_exists('right_delimiter', $config)) {
			
			$this->_smarty->right_delimiter = $config['right_delimiter'];
		}
		$this->assign('this', $this);
		
		//		$this->_smarty = new Hpls_Smarty_Smarty();
		//		$this->_smarty->plugins_dir = array('plugins', APPLIANCE_PATH. '/libs/Hpls/Smarty/Plugins');
		//		$this->setScriptPath($config['scriptPath']);
		// Ścieżki do helperów
		//		foreach ($config['helperDirs'] as $path => $prefix) {
		//			$this->addHelperPath($path, $prefix);
		//		}
		

		if (isDebug('smarty')) {
			$this->_smarty->debugging = TRUE;
		} else {
			$this->_smarty->debugging = FALSE;
		}
	
		//		Hpls_Debug::log()->debug(Hpls_Debug::print_r($this->_smarty, "SMARTY"));
	}

	/**
	 * Implement _run() method
	 *
	 * The method _run() is the only method that needs to be implemented in any subclass of Zend_View_Abstract. It is called automatically within the render() method. My implementation just uses the display() method from Smarty to generate and output the template.
	 *
	 * @param string $template
	 */
	protected function _run() {
		
		$file = func_num_args() > 0 && file_exists(func_get_arg(0)) ? func_get_arg(0) : '';
		Zend_Registry::get(Hpls_Consts::REGK_LOGGER)->debug("Rendering template: " . $file);
		if ($this->_customTemplate || $file) {
			$template = $this->_customTemplate;
			if (!$template) {
				$template = $file;
			}
			
			$this->_smarty->display($template);
		} else {
			throw new Zend_View_Exception('Cannot render view without any template being assigned or file does not exist');
		}
	}
}